#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CES Alarm v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _translate_alarm_level(level):
    case = {
        1: '1: Critical',
        2: '2: Major',
        3: '3: Minor',
        4: '4: Informational'
    }
    return case.get(level)


def _flatten_output(obj):
    data = {
        'id': obj.id,
        'name': obj.name,
        'namespace': obj.metric.namespace,
        # The return value of obj.metric.dimensions is a list. The list has
        # only one value. It is not possible to have several items inside.
        'dimensions.name': obj.metric.dimensions[0].name,
        'dimensions.value': obj.metric.dimensions[0].value,
        'alarm_level': _translate_alarm_level(obj.alarm_level),
        'enabled': obj.alarm_enabled,
        'action_enabled': obj.alarm_action_enabled,
        'state': obj.alarm_state,
    }
    return data


def _get_columns(item):
    column_map = {
    }
    inv_columns = ['']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           inv_columns)


# TODO(undefined): Implement query arguments -> SDK not working
class ListAlarms(command.Lister):
    _description = _('List CES alarms')
    columns = (
        'id',
        'name',
        'namespace',
        'dimensions.name',
        'dimensions.value',
        'alarm_level',
        'enabled',
        'action_enabled',
        'state'
    )

    def get_parser(self, prog_name):
        parser = super(ListAlarms, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.ces

        data = client.alarms()

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_output(s), self.columns
                 ) for s in data))
        return table


class ShowAlarm(command.ShowOne):
    _description = _('Show CloudEye alarm rule details')

    def get_parser(self, prog_name):
        parser = super(ShowAlarm, self).get_parser(prog_name)

        parser.add_argument(
            'alarm',
            metavar='<alarm>',
            help=_('UUID or name of the alarm rule.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.ces

        obj = client.find_alarm(
            parsed_args.alarm,
            ignore_missing=False
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteAlarm(command.Command):
    _description = _('Delete CES alarm')

    def get_parser(self, prog_name):
        parser = super(DeleteAlarm, self).get_parser(prog_name)

        parser.add_argument(
            'alarm',
            metavar='<alarm>',
            nargs='+',
            help=_('UUID or name of the alarm.')
        )

        return parser

    def take_action(self, parsed_args):
        if parsed_args.alarm:
            client = self.app.client_manager.ces
            for alarm in parsed_args.alarm:
                alarm = client.find_alarm(alarm, ignore_missing=False)
                client.delete_alarm(alarm=alarm)


class SetAlarm(command.ShowOne):
    _description = _('Switch Alarm status.')

    def get_parser(self, prog_name):
        parser = super(SetAlarm, self).get_parser(prog_name)

        parser.add_argument(
            'alarm',
            metavar='<alarm>',
            help=_('UUID or name of the alarm.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.ces

        alarm = client.find_alarm(parsed_args.alarm, ignore_missing=False)

        if alarm:
            client.switch_alarm_state(
                alarm=alarm
            )

            # instance of alarm needs to be found again due to missing
            # return body of alarm rule update function
            obj = client.find_alarm(
                parsed_args.alarm,
                ignore_missing=False
            )
            display_columns, columns = _get_columns(obj)
            data = utils.get_item_properties(obj, columns)
            return (display_columns, data)


class CreateAlarm(command.ShowOne):
    _description = _('Create CloudEye alarm rule')

    def get_parser(self, prog_name):
        parser = super(CreateAlarm, self).get_parser(prog_name)

        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Alarm name')
        )
        parser.add_argument(
            '--enabled',
            metavar='<enabled>',
            default=True,
            type=bool,
            help=_('State of the alarm.\n'
                   'True: enable alarm (default)\n'
                   'False: disable alarm\n')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description of the alarm')
        )
        parser.add_argument(
            '--action-enabled',
            default=False,
            type=bool,
            help=_('Specifies whether the alarm action is triggered')
        )
        parser.add_argument(
            '--level',
            metavar='<level>',
            type=int,
            help=_('Indicates the alarm level\n'
                   '1: critical\n'
                   '2: major\n'
                   '3: minor\n'
                   '4: informational')
        )

        # AlarmActions
        parser.add_argument(
            '--alarm-action-type',
            metavar='<alarm_action_type>',
            help=_('Specifies the alarms action type.\n'
                   'notification: notification will be sent to user\n'
                   'autoscaling: scaling action will be triggered')
        )
        parser.add_argument(
            '--alarm-action-notification-list',
            metavar='<alarm_action_notification_list>',
            action='append',
            help=_('Specifies the list of objects being notified when '
                   'alarm status changes.\n'
                   'URN example structure:\n'
                   'urn:smn:region:68438a86d98e427e907e0097b7e35d48:sd\n'
                   'The parameter can be given multiple times to '
                   'notify multiple targets.')
        )

        # OkActions
        parser.add_argument(
            '--ok-action-type',
            metavar='<ok_action_type>',
            help=_('Specifies the alarms action type.\n'
                   'notification: notification will be sent to user\n'
                   'autoscaling: scaling action will be triggered')
        )
        parser.add_argument(
            '--ok-action-notification-list',
            metavar='<ok_action_notification_list>',
            action='append',
            help=_('Specifies the list of objects being notified when '
                   'alarm status changes.\n'
                   'URN example structure:\n'
                   'urn:smn:region:68438a86d98e427e907e0097b7e35d48:sd\n'
                   'The parameter can be given multiple times to '
                   'notify multiple targets.')
        )

        # ConditionSpec
        parser.add_argument(
            '--comparison-operator',
            metavar='<comparison_operator>',
            required=True,
            help=_('Specifies the conditions comparison operator')
        )
        parser.add_argument(
            '--count',
            metavar='<count>',
            type=int,
            required=True,
            help=_('Specifies how many times the alarm condition has to '
                   'triggered until Alarm raises.\n'
                   'Value range: 1 to 5')
        )
        parser.add_argument(
            '--filter',
            metavar='<filter>',
            required=True,
            help=_('Specifies the data rollup method.\n'
                   'Values: max, min, average, sum, variance')
        )
        parser.add_argument(
            '--period',
            metavar='<period>',
            type=int,
            required=True,
            help=_('Indicates the interval (in seconds) for checking '
                   'whether the configured alarm rules are met.')
        )
        parser.add_argument(
            '--unit',
            metavar='<unit>',
            help=_('Specifies data unit.')
        )
        parser.add_argument(
            '--value',
            metavar='<value>',
            type=int,
            required=True,
            help=_('Specifies the alarm threshold.\n'
                   'Values: 0 to max(int)')
        )

        # DimensionsSpec for Metrics
        parser.add_argument(
            '--dimension',
            metavar='name=<dimension-name>,value=<dimensions-value>',
            action=parseractions.MultiKeyValueAction,
            dest='dimensions',
            required=True,
            required_keys=['name', 'value'],
            help=_('Example: \n'
                   '--dimension name=instance_id,value=123456-bfdba93d4123\n'
                   'Repeat option to provide multiple dimensions.')
        )

        # MetricSpec
        parser.add_argument(
            '--metric-name',
            metavar='<metric_name>',
            required=True,
            help=_('Specifies the metric name')
        )
        parser.add_argument(
            '--namespace',
            metavar='<namespace>',
            required=True,
            help=_('Specifies the namespace of the metric such as:\n'
                   'SYS.ECS, SYS.AS')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.ces

        attrs = {}

        attrs['name'] = parsed_args.name
        if parsed_args.enabled:
            attrs['alarm_enabled'] = parsed_args.enabled
        if parsed_args.description:
            attrs['alarm_description'] = parsed_args.description
        attrs['alarm_action_enabled'] = parsed_args.action_enabled
        if parsed_args.level:
            attrs['alarm_level'] = parsed_args.level

        ok_actions = []
        alarm_actions = []

        if parsed_args.action_enabled:
            if (parsed_args.ok_action_type
                    and parsed_args.ok_action_notification_list):
                nl = parsed_args.ok_action_notification_list
                ok_actions.append({
                    'type': parsed_args.ok_action_type,
                    'notificationList': nl
                })
                attrs['ok_actions'] = ok_actions

            if (parsed_args.alarm_action_type
                    and parsed_args.alarm_action_notification_list):

                nl = parsed_args.alarm_action_notification_list
                alarm_actions.append({
                    'type': parsed_args.alarm_action_type,
                    'notificationList': nl
                })
                attrs['alarm_actions'] = alarm_actions

        condition = {
            'comparison_operator': parsed_args.comparison_operator,
            'count': parsed_args.count,
            'filter': parsed_args.filter,
            'period': parsed_args.period,
            'value': parsed_args.value
        }
        if parsed_args.unit:
            condition['unit'] = parsed_args.unit
        attrs['condition'] = condition

        metric = {
            'dimensions': parsed_args.dimensions,
            'metric_name': parsed_args.metric_name,
            'namespace': parsed_args.namespace
        }
        attrs['metric'] = metric

        obj = client.create_alarm(
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class CbcOperationLockSpec(resource.Resource):
    lock_scene = resource.Body('lock_scene')
    lock_source_id = resource.Body('lock_source_id')


class TmsKeyValueSpec(resource.Resource):
    key = resource.Body('key')
    value = resource.Body('value')


class EndPointServiceSpec(resource.Resource):
    service_name = resource.Body('service_name')
    created_at = resource.Body('created_at')


class NodeIpsSpec(resource.Resource):
    livedata = resource.Body('livedata')
    shubao = resource.Body('shubao')


class IpSpec(resource.Resource):
    ip_address = resource.Body('ip_address')
    bandwidth_size = resource.Body('bandwidth_size', type=int)


class Gateway(resource.Resource):
    base_path = '/apigw/instances'

    allow_list = True
    allow_fetch = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    _query_mapping = resource.QueryParameters('limit', 'offset', 'instance_id',
                                              'instance_name', 'status')

    # Properties
    description = resource.Body('description')
    maintain_begin = resource.Body('maintain_begin')
    maintain_end = resource.Body('maintain_end')
    instance_name = resource.Body('instance_name')
    security_group_id = resource.Body('security_group_id')
    vpcep_service_name = resource.Body('vpcep_service_name')

    # Attributes
    id = resource.Body('id')
    project_id = resource.Body('project_id')
    status = resource.Body('status')
    instance_status = resource.Body('instance_status', type=int)
    type = resource.Body('type')
    spec = resource.Body('spec')
    create_time = resource.Body('create_time')
    enterprise_project_id = resource.Body('enterprise_project_id')
    eip_address = resource.Body('eip_address')
    charging_mode = resource.Body('charging_mode', type=int)
    cbc_metadata = resource.Body('cbc_metadata', type=int)
    loadbalancer_provider = resource.Body('loadbalancer_provider')
    cbc_operation_locks = resource.Body('cbc_operation_locks', type=list,
                                        list_type=CbcOperationLockSpec)
    instance_id = resource.Body('instance_id')
    spec_id = resource.Body('spec_id')
    vpc_id = resource.Body('vpc_id')
    subnet_id = resource.Body('subnet_id')
    eip_id = resource.Body('eip_id')
    available_zone_ids = resource.Body('available_zone_ids', type=list)
    bandwidth_size = resource.Body('bandwidth_size', type=int)
    bandwidth_charging_mode = resource.Body('bandwidth_charging_mode')
    tags = resource.Body('tags', type=list, list_type=TmsKeyValueSpec)
    ingress_bandwidth_size = resource.Body('ingress_bandwidth_size', type=int)
    ingress_bandwidth_charging_mode = resource.Body(
        'ingress_bandwidth_charging_mode')
    message = resource.Body('message')
    job_id = resource.Body('job_id')
    ingress_ip = resource.Body('ingress_ip')
    ingress_ip_v6 = resource.Body('ingress_ip_v6')
    user_id = resource.Body('user_id')
    nat_eip_address = resource.Body('nat_eip_address')
    instance_version = resource.Body('instance_version')
    virsubnet_id = resource.Body('virsubnet_id')
    roma_eip_address = resource.Body('roma_eip_address')
    supported_features = resource.Body('supported_features', type=list)
    endpoint_service = resource.Body('endpoint_service',
                                     type=EndPointServiceSpec)
    endpoint_services = resource.Body('endpoint_services', type=list,
                                      list_type=EndPointServiceSpec)
    node_ips = resource.Body('node_ips', type=NodeIpsSpec)
    publicips = resource.Body('publicips', type=IpSpec)
    privateips = resource.Body('privateips', type=IpSpec)
    unreliable_ips = resource.Body('unreliable_ips', type=list)
    is_releasable = resource.Body('is_releasable', type=bool)
    progress = resource.Body('progress', type=int)
    error_code = resource.Body('error_code')
    error_msg = resource.Body('error_msg')
    start_time = resource.Body('start_time')
    end_time = resource.Body('end_time')
    listeners = resource.Body('listeners', type=dict)
    restrict_cidrs = resource.Body('restrict_cidrs', type=list)
    resource_subnet_cidr = resource.Body('resource_subnet_cidr')
    cloud_eip_id = resource.Body("cloudEipId")
    cloud_eip_address = resource.Body("cloudEipAddress")
    cloud_bandwidth_id = resource.Body('cloudBandwidthId')
    bandwidth_name = resource.Body('bandwidthName')
    charge_mode = resource.Body('chargeMode')

    def _get_creation_progress(self, session, gateway):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/progress'
        response = session.get(url)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _get_constraints(self, session, gateway):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/restriction'
        response = session.get(url)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _modify_spec(self, session, gateway, **attrs):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/postpaid-resize'
        response = session.post(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _bind_eip(self, session, gateway, **attrs):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/eip'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _unbind_eip(self, session, gateway):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/eip'
        response = session.delete(url)
        exceptions.raise_from_response(response)
        return None

    def _enable_public_access(self, session, gateway, **attrs):
        return self._enable_eip(session, gateway, 'nat-eip', **attrs)

    def _update_public_access(self, session, gateway, **attrs):
        return self._update_eip(session, gateway, 'nat-eip', **attrs)

    def _disable_public_access(self, session, gateway):
        return self._delete_eip(session, gateway, 'nat-eip')

    def _enable_ingress(self, session, gateway, **attrs):
        return self._enable_eip(session, gateway, 'ingress-eip', **attrs)

    def _update_ingress(self, session, gateway, **attrs):
        return self._update_eip(session, gateway, 'ingress-eip', **attrs)

    def _disable_ingress(self, session, gateway):
        return self._delete_eip(session, gateway, 'ingress-eip')

    def _enable_eip(self, session, gateway, url, **attrs):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/{url}'
        response = session.post(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _update_eip(self, session, gateway, url, **attrs):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/{url}'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _delete_eip(self, session, gateway, url):
        gw_id = gateway.instance_id if gateway.id is None else gateway.id
        url = f'{self.base_path}/{gw_id}/{url}'
        response = session.delete(url)
        exceptions.raise_from_response(response)
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import custom_authorizer

EXAMPLE_CUSTOM_AUTHORIZER = {
    'gateway_id': 'gateway-67890',
    'id': 'auth-001',
    'name': 'My Custom Authorizer',
    'type': 'JWT',
    'authorizer_type': 'FUNC',
    'authorizer_uri': 'https://example.com/auth',
    'network_type': 'VPC',
    'authorizer_version': 'v1',
    'authorizer_alias_uri': 'https://alias.example.com/auth',
    'identities': [
        {'name': 'Authorization', 'location': 'HEADER', 'validation': '.*'}
    ],
    'ttl': 300,
    'user_data': 'custom user data',
    'ld_api_id': 'api-12345',
    'need_body': True,
    'create_time': '2025-04-01T10:30:00Z',
    'roma_app_id': 'roma-999',
    'roma_app_name': 'My ROMA App'
}


class TestCustomAuthorizer(base.TestCase):

    def test_basic(self):
        sot = custom_authorizer.CustomAuthorizer()
        self.assertEqual('/apigw/instances/%(gateway_id)s/authorizers',
                         sot.base_path)
        self.assertEqual('authorizer_list', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_patch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = custom_authorizer.CustomAuthorizer(**EXAMPLE_CUSTOM_AUTHORIZER)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['gateway_id'],
                         sot.gateway_id)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['id'], sot.id)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['name'], sot.name)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['type'], sot.type)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['authorizer_type'],
                         sot.authorizer_type)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['authorizer_uri'],
                         sot.authorizer_uri)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['network_type'],
                         sot.network_type)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['authorizer_version'],
                         sot.authorizer_version)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['authorizer_alias_uri'],
                         sot.authorizer_alias_uri)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['ttl'], sot.ttl)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['user_data'], sot.user_data)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['ld_api_id'], sot.ld_api_id)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['need_body'], sot.need_body)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['create_time'],
                         sot.create_time)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['roma_app_id'],
                         sot.roma_app_id)
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['roma_app_name'],
                         sot.roma_app_name)
        self.assertEqual(len(EXAMPLE_CUSTOM_AUTHORIZER['identities']),
                         len(sot.identities))
        self.assertEqual(EXAMPLE_CUSTOM_AUTHORIZER['identities'][0]['name'],
                         sot.identities[0].name)
        self.assertEqual(
            EXAMPLE_CUSTOM_AUTHORIZER['identities'][0]['location'],
            sot.identities[0].location)
        self.assertEqual(
            EXAMPLE_CUSTOM_AUTHORIZER['identities'][0]['validation'],
            sot.identities[0].validation)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import random
import uuid

import mock

from openstack.network.v2 import health_monitor
from openstack.network.v2 import listener
from openstack.network.v2 import load_balancer
from openstack.network.v2 import pool
from openstack.network.v2 import pool_member

from otcextensions.tests.unit.osclient import test_base


class TestLoadBalancer(test_base.TestCommand):

    def setUp(self):
        super(TestLoadBalancer, self).setUp()

        self.app.client_manager.network = mock.Mock()
        self.client = self.app.client_manager.network


class FakeLoadBalancer(test_base.Fake):
    """Fake one or more LB"""

    @classmethod
    def generate(cls):
        object_info = {
            'admin_state_up': True,
            'description': 'descr-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'listeners': [{'id': 'id-' + uuid.uuid4().hex}],
            'name': 'name-' + uuid.uuid4().hex,
            'operating_status': 'op_stat-' + uuid.uuid4().hex,
            'provisioning_status': 'ps-' + uuid.uuid4().hex,
            'project_id': 'ten-' + uuid.uuid4().hex,
            'vip_address': 'vip_addr-' + uuid.uuid4().hex,
            'vip_subnet_id': 'vip_subn-' + uuid.uuid4().hex,
            'vip_port_id': 'vip_port-' + uuid.uuid4().hex,
            'provider': 'provider-' + uuid.uuid4().hex,
            'pools': [{'id': 'id-' + uuid.uuid4().hex}],
        }
        obj = load_balancer.LoadBalancer.existing(**object_info)
        return obj


class FakeListener(test_base.Fake):
    """Fake one or more LB Listener"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'admin_state_up': True,
            'description': 'descr-' + uuid.uuid4().hex,
            'loadbalancers': [{'id': 'id-' + uuid.uuid4().hex}],
            'protocol': 'proto-' + uuid.uuid4().hex,
            'protocol_port': random.randint(1, 65535),
            'default_pool_id': 'poolid-' + uuid.uuid4().hex,
            'connection_limit': random.randint(-1, 10),

        }
        obj = listener.Listener.existing(**object_info)
        return obj


class FakePool(test_base.Fake):
    """Fake one or more LB Pool"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'admin_state_up': True,
            'description': 'descr-' + uuid.uuid4().hex,
            'loadbalancers': [{'id': 'id-' + uuid.uuid4().hex}],
            'listeners': [{'id': 'id-' + uuid.uuid4().hex}],
            'protocol': 'proto-' + uuid.uuid4().hex,
            'session_persistence': True,
            'healthmonitor_id': 'hmid-' + uuid.uuid4().hex,
            # 'health_monitor_ids': [{'id': 'hmid-' + uuid.uuid4().hex}],
            'member_ids': [{'id': 'id-' + uuid.uuid4().hex}],
            'lb_algorithm': 'algo-' + uuid.uuid4().hex,
            'status': 'dummy'

        }
        obj = pool.Pool.existing(**object_info)
        return obj


class FakePoolMember(test_base.Fake):
    """Fake one or more LB Pool Member"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'admin_state_up': True,
            'address': 'addr-' + uuid.uuid4().hex,
            'operating_status': 'stat-' + uuid.uuid4().hex,
            'pool_id': 'pool_id-' + uuid.uuid4().hex,
            'protocol_port': random.randint(1, 65535),
            'weight': random.randint(1, 65535),
            'subnet_id': 'subnet-' + uuid.uuid4().hex,
        }
        obj = pool_member.PoolMember.existing(**object_info)
        return obj


class FakeHealthMonitor(test_base.Fake):
    """Fake one or more LB Health monitor"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'admin_state_up': True,
            'type': 'type-' + uuid.uuid4().hex,
            'http_method': 'method-' + uuid.uuid4().hex,
            'url_path': 'url_path-' + uuid.uuid4().hex,
            'pools': [{'id': 'id-' + uuid.uuid4().hex}],
            'max_retries': random.randint(1, 65535),
            'timeout': random.randint(1, 65535),
            'expected_codes': 'codes-' + uuid.uuid4().hex,

        }
        obj = health_monitor.HealthMonitor.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils


class Resource(resource.Resource):

    query_marker_key = 'start_number'

    _query_mapping = resource.QueryParameters(
        'marker', 'limit',
        marker='start_number',
        limit='limit'
    )

    @classmethod
    def _get_next_link(cls, uri, response, data, marker, limit, total_yielded):
        # AS service pagination. Returns query for the next page
        next_link = None
        params = {}
        if total_yielded < data['total_number']:
            next_link = uri
            params['marker'] = total_yielded
            params['limit'] = limit
        else:
            next_link = None
        query_params = cls._query_mapping._transpose(params, cls)
        return next_link, query_params

    @staticmethod
    def find_value_by_accessor(input_dict, accessor):
        """Gets value from a dictionary using a dotted accessor"""
        current_data = input_dict
        for chunk in accessor.split('.'):
            if isinstance(current_data, dict):
                current_data = current_data.get(chunk, {})
            else:
                return None
        return current_data

    def _action(self, session, body):
        """Preform alarm actions given the message body.

        """
        # if getattr(self, 'endpoint_override', None):
        #     # If we have internal endpoint_override - use it
        #     endpoint_override = self.endpoint_override
        url = utils.urljoin(self.base_path, self.id, 'action')
        return session.post(
            url,
            # endpoint_override=endpoint_override,
            json=body)

    def commit(self, session, prepend_key=False, has_body=True,
               retry_on_conflict=None, base_path=None):
        return \
            super(Resource, self).commit(session, prepend_key=prepend_key,
                                         has_body=has_body,
                                         retry_on_conflict=retry_on_conflict,
                                         base_path=base_path)

    def create(self, session, prepend_key=False, base_path=None, **params):
        return super(Resource, self).create(session, prepend_key=prepend_key,
                                            base_path=base_path, **params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""DIS Stream v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils


LOG = logging.getLogger(__name__)


_formatters = {}


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
        'id',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


CHECKPOINT_TYPE_CHOICES = ('LAST_READ',)


class ShowCheckpoint(command.ShowOne):
    _description = _("Querying Checkpoint Details.")

    def get_parser(self, prog_name):
        parser = super(ShowCheckpoint, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of the stream to which the checkpoint belongs."),
        )
        parser.add_argument(
            'appName',
            metavar='<appName>',
            help=_("Name of the app associated with the checkpoint."),
        )
        parser.add_argument(
            '--partition-id',
            metavar='<partition_id>',
            required=True,
            help=_("Identifier of the stream partition to which the "
                   "checkpoint belongs."),
        )
        parser.add_argument(
            '--checkpoint-type',
            dest='checkpoint_type',
            metavar='{' + ','.join(CHECKPOINT_TYPE_CHOICES) + '}',
            default='LAST_READ',
            type=lambda s: s.upper(),
            choices=CHECKPOINT_TYPE_CHOICES,
            help=_("Type of the checkpoint."
                   "\nLAST_READ: Only sequence numbers are recorded in"
                   "databases."
                   "\nDefault: LAST_READ"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        params = {
            'stream_name': parsed_args.streamName,
            'app_name': parsed_args.appName,
            'partition_id': parsed_args.partition_id,
            'checkpoint_type': parsed_args.checkpoint_type
        }

        obj = client.get_checkpoint(**params)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateCheckpoint(command.ShowOne):
    _description = _("Add Checkpoint.")

    def get_parser(self, prog_name):
        parser = super(CreateCheckpoint, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of the stream to which the checkpoint belongs."),
        )
        parser.add_argument(
            'appName',
            metavar='<appName>',
            help=_("Name of the app associated with the checkpoint."),
        )
        parser.add_argument(
            '--partition-id',
            metavar='<partition_id>',
            required=True,
            help=_("Identifier of the stream partition to which the "
                   "checkpoint belongs."),
        )
        parser.add_argument(
            '--sequence-number',
            metavar='<sequence_number>',
            required=True,
            help=_("Sequence number to be submitted, which is used to "
                   "record the consumption checkpoint of the stream."),
        )
        parser.add_argument(
            '--metadata',
            metavar='<metadata>',
            help=_("Metadata information of the consumer application. "
                   "The metadata information can contain a maximum of "
                   "1,000 characters."),
        )

        parser.add_argument(
            '--checkpoint-type',
            dest='checkpoint_type',
            metavar='{' + ','.join(CHECKPOINT_TYPE_CHOICES) + '}',
            default='LAST_READ',
            type=lambda s: s.upper(),
            choices=CHECKPOINT_TYPE_CHOICES,
            help=_("Type of the checkpoint."
                   "\nLAST_READ: Only sequence numbers are recorded in"
                   "databases."
                   "\nDefault: LAST_READ"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        attrs = {
            'stream_name': parsed_args.streamName,
            'app_name': parsed_args.appName,
            'partition_id': parsed_args.partition_id,
            'sequence_number': parsed_args.sequence_number,
            'checkpoint_type': parsed_args.checkpoint_type
        }
        if parsed_args.metadata:
            attrs.update(metadata=parsed_args.metadata)

        obj = client.create_checkpoint(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteCheckpoint(command.Command):
    _description = _("Delete Checkpoint.")

    def get_parser(self, prog_name):
        parser = super(DeleteCheckpoint, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of the stream to which the checkpoint belongs."),
        )
        parser.add_argument(
            'appName',
            metavar='<appName>',
            help=_("Name of the app associated with the checkpoint."),
        )
        parser.add_argument(
            '--partition-id',
            metavar='<partition_id>',
            help=_("Identifier of the stream partition to which the "
                   "checkpoint belongs."),
        )
        parser.add_argument(
            '--checkpoint-type',
            dest='checkpoint_type',
            metavar='{' + ','.join(CHECKPOINT_TYPE_CHOICES) + '}',
            default='LAST_READ',
            type=lambda s: s.upper(),
            choices=CHECKPOINT_TYPE_CHOICES,
            help=_("Type of the checkpoint."
                   "\nLAST_READ: Only sequence numbers are recorded in"
                   "databases."
                   "\nDefault: LAST_READ"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        params = {
            'stream_name': parsed_args.streamName,
            'app_name': parsed_args.appName,
            'checkpoint_type': parsed_args.checkpoint_type
        }
        if parsed_args.partition_id:
            params.update(partition_id=parsed_args.partition_id)

        client.delete_checkpoint(**params)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create CCE node pool
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    'metadata': {
        'name': 'test-node-pool'
    },
    'spec': {
        'initialNodeCount': 0,
        'type': 'vm',
        'autoscaling': {
            'enable': True,
            'minNodeCount': 1,
            'maxNodeCount': 3,
            'scaleDownCooldownTime': 10,
            'priority': 1
        },
        'nodeTemplate': {
            'flavor': 's2.large.2',
            'az': 'eu-de-01',
            'os': 'CentOS 7.7',
            'login': {
                'sshKey': 'sshkey-pub'
            },
            'rootVolume': {
                'volumetype': 'SATA',
                'size': 40
            },
            'dataVolumes': [
                {
                    'volumetype': 'SATA',
                    'size': 100,
                    'extendParam': {
                        'useType': 'docker'
                    }
                }
            ],
            'billingMode': 0,
            'extendParam': {
                'maxPods': 110,
                'DockerLVMConfigOverride': 'dockerThinpool=vgpaas/90%VG;'
                                           'kubernetesLV=vgpaas/10%VG;'
                                           'diskType=evs;lvType=linear'
            },
            'k8sTags': {
                'tag1': 'value1',
                'tag2': 'value2'
            },
            'taints': [
                {
                    'key': 'aaa',
                    'value': 'bbb',
                    'effect': 'NoSchedule'
                },
                {
                    'key': 'ccc',
                    'value': 'ddd',
                    'effect': 'NoSchedule'
                }
            ],
            'userTags': [
                {
                    'key': 'resource-tag1',
                    'value': 'value1'
                },
                {
                    'key': 'resource-tag2',
                    'value': 'value2'
                }
            ],
            'nodeNicSpec': {
                'primaryNic': {
                    'subnetId': 'subnet_id'
                }
            }
        }
    }
}
cluster = 'name_or_id'
cluster = conn.cce.find_cluster(name_or_id=cluster)
conn.cce.create_node_pool(cluster=cluster, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.ces.v1 import _proxy
from otcextensions.sdk.ces.v1 import alarm
from otcextensions.sdk.ces.v1 import event_data
from otcextensions.sdk.ces.v1 import metric
from otcextensions.sdk.ces.v1 import metric_data
from otcextensions.sdk.ces.v1 import quota


class TestCesProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestCesProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestCesAlarm(TestCesProxy):
    def test_alarm_create(self):
        self.verify_create(self.proxy.create_alarm, alarm.Alarm,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_alarm_delete(self):
        self.verify_delete(self.proxy.delete_alarm,
                           alarm.Alarm, True)

    def test_alarm_get(self):
        self.verify_get(self.proxy.get_alarm, alarm.Alarm)

    def test_alarms(self):
        self.verify_list(self.proxy.alarms, alarm.Alarm)

    def test_update_alarm_enabled(self):
        self._verify(
            'otcextensions.sdk.ces.v1.alarm.Alarm.change_alarm_status',
            self.proxy.switch_alarm_state,
            method_args=["alarm"],
            expected_args=[self.proxy]
        )

    def test_alarm_find(self):
        self.verify_find(self.proxy.find_alarm, alarm.Alarm)


class TestCesEventData(TestCesProxy):
    def test_event_data(self):
        self.verify_list(self.proxy.event_data, event_data.EventData)


class TestCesMetric(TestCesProxy):
    def test_metrics(self):
        self.verify_list(self.proxy.metrics, metric.Metric)


class TestCesMetricData(TestCesProxy):
    def test_metric_data(self):
        self.verify_list(self.proxy.metric_data, metric_data.MetricData)


class TestCesQuota(TestCesProxy):
    def test_quotas(self):
        self.verify_list(self.proxy.quotas, quota.Quota)


class TestExtractName(TestCesProxy):

    def test_extract_name(self):

        self.assertEqual(
            ['discovery'],
            self.proxy._extract_name(
                '/', project_id='123')
        )
        self.assertEqual(
            ['discovery'],
            self.proxy._extract_name(
                '/V1.0/', project_id='123')
        )
        self.assertEqual(
            ['alarms'],
            self.proxy._extract_name(
                '/V1.0/123/alarms', project_id='123')
        )
        self.assertEqual(
            ['alarm'],
            self.proxy._extract_name(
                '/V1.0/123/alarms/some_id', project_id='123')
        )
        self.assertEqual(
            ['alarm', 'action'],
            self.proxy._extract_name(
                '/V1.0/123/alarms/some_id/action', project_id='123')
        )
        self.assertEqual(
            ['metrics'],
            self.proxy._extract_name(
                '/V1.0/123/metrics', project_id='123')
        )
        self.assertEqual(
            ['metric-data'],
            self.proxy._extract_name(
                '/V1.0/123/metric-data?a=b', project_id='123')
        )
        self.assertEqual(
            ['batch-query-metric-data'],
            self.proxy._extract_name(
                '/V1.0/123/batch-query-metric-data', project_id='123')
        )
        self.assertEqual(
            ['event-data'],
            self.proxy._extract_name(
                '/V1.0/123/event-data', project_id='123')
        )
        self.assertEqual(
            ['quotas'],
            self.proxy._extract_name(
                '/V1.0/123/quotas', project_id='123')
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''LoadBalancer Pool v1 action implementations'''
import json
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


_formatters = {
    'health_monitor_ids': sdk_utils.ListOfIdsColumnBR,
    'listener_ids': sdk_utils.ListOfIdsColumnBR,
    'load_balancer_ids': sdk_utils.ListOfIdsColumnBR,
    'member_ids': sdk_utils.ListOfIdsColumnBR,
}


LB_ALGORITHM_VALUES = ['LEAST_CONNECTIONS', 'ROUND_ROBIN', 'SOURCE_IP']
PROTOCOL_VALUES = ['HTTP', 'HTTPS', 'PROXY', 'TCP', 'UDP']


def _get_columns(item):
    column_map = {
        'is_admin_state_up': 'admin_state_up',
        'load_balancer_ids': 'loadbalancers',
        'listener_ids': 'listeners',
        'status': 'provisioning_status',
        'health_monitor_id': 'healthmonitor_id'
        # 'listeners': 'listener_ids',
        # 'pools': 'pool_ids',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListPool(command.Lister):
    _description = _('List LoadBalancer pools')
    column_headers = (
        'id', 'name', 'project_id', 'provisioning_status', 'protocol',
        'lb_algorithm', 'admin_state_up')
    columns = (
        'id', 'name', 'project_id', 'provisioning_status', 'protocol',
        'lb_algorithm', 'is_admin_state_up')

    def get_parser(self, prog_name):
        parser = super(ListPool, self).get_parser(prog_name)

        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Load balancer pool description to query')
        )
        parser.add_argument(
            '--lb_algorithm',
            metavar='{' + ','.join(LB_ALGORITHM_VALUES) + '}',
            type=lambda s: s.upper(),
            choices=LB_ALGORITHM_VALUES,
            help=_('Load balancer pool algorithm to query'
                   'one of [`LEAST_CONNECTIONS`, `ROUND_ROBIN`, `SOURCE_IP`]')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Load balancer pool name to query')
        )
        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(PROTOCOL_VALUES) + '}',
            type=lambda s: s.upper(),
            choices=PROTOCOL_VALUES,
            help=_('Load balancer pool protocol to query'
                   'one of [`HTTP`, `HTTPS`, `PROXY`, `TCP`, `UDP`]')
        )
        parser.add_argument(
            '--load_balancer',
            metavar='<load_balancer>',
            help=_('Filter by load balancer (name or ID).')
        )
        # parser.add_argument(
        #     '--listener',
        #     metavar='<listener>',
        #     help=_('Filter by listener (name or ID).')
        # )
        return parser

    def take_action(self, parsed_args):

        args = {}
        if parsed_args.description:
            args['description'] = parsed_args.description
        if parsed_args.lb_algorithm:
            args['lb_algorithm'] = parsed_args.lb_algorithm
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.load_balancer:
            args['load_balancer_id'] = parsed_args.load_balancer
        if parsed_args.protocol:
            args['protocol'] = parsed_args.protocol
        if parsed_args.lb_algorithm:
            args['lb_algorithm'] = parsed_args.lb_algorithm
        # if parsed_args.listener:
        #     args['listener_id'] = parsed_args.listener

        client = self.app.client_manager.network

        data = client.pools(**args)

        return (
            self.column_headers,
            (utils.get_item_properties(
                s, self.columns, formatters=_formatters
            ) for s in data))


class ShowPool(command.ShowOne):
    _description = _('Show LoadBalancer pool details')

    def get_parser(self, prog_name):
        parser = super(ShowPool, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            help=_('Load balancer pool id to show.')
        )

        return parser

    def take_action(self, parsed_args):

        args = {}

        client = self.app.client_manager.network

        obj = client.find_pool(
            name_or_id=parsed_args.pool,
            ignore_missing=False,
            **args
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreatePool(command.ShowOne):
    _description = _('Create a pool')

    def get_parser(self, prog_name):
        parser = super(CreatePool, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Set pool name.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Set pool description.')
        )
        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(PROTOCOL_VALUES) + '}',
            type=lambda s: s.upper(),
            choices=PROTOCOL_VALUES,
            required=True,
            help=_('The protocol for the pool. '
                   'One of [`HTTP`, `HTTPS`, `PROXY`, `TCP`, `UDP`].')
        )
        parser.add_argument(
            '--lb_algorithm',
            metavar='{' + ','.join(LB_ALGORITHM_VALUES) + '}',
            type=lambda s: s.upper(),
            choices=LB_ALGORITHM_VALUES,
            required=True,
            help=_('The load balancing algorithm for the pool. '
                   'One of [`LEAST_CONNECTIONS`, `ROUND_ROBIN`, `SOURCE_IP`].')
        )
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            '--listener_id',
            metavar='<listener>',
            help=_('The ID of the listener for the pool. '
                   'Either listener_id or loadbalancer_id must be specified.')
        )
        group.add_argument(
            '--loadbalancer_id',
            metavar='<loadbalancer>',
            help=_('The ID of the loadbalancer for the pool. '
                   'Either listener_id or loadbalancer_id must be specified.')
        )
        parser.add_argument(
            '--session_persistence',
            metavar='<session_persistence>',
            help=_('A JSON object specifying the session persistence '
                   'for the pool or null for no session persistence. '
                   'See Pool Session Persistence. Default is null.')
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help=_('Enable pool (default).')
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help=_('Disable pool.')
        )

        return parser

    def take_action(self, parsed_args):

        args = {}

        if parsed_args.protocol:
            args['protocol'] = parsed_args.protocol
        if parsed_args.lb_algorithm:
            args['lb_algorithm'] = parsed_args.lb_algorithm
        if parsed_args.listener_id is not None:
            args['listener_id'] = parsed_args.listener_id
        if parsed_args.loadbalancer_id is not None:
            args['loadbalancer_id'] = parsed_args.loadbalancer_id
        if parsed_args.disable is not None:
            args['is_admin_state_up'] = False
        if parsed_args.name is not None:
            args['name'] = parsed_args.name
        if parsed_args.description is not None:
            args['description'] = parsed_args.description
        if parsed_args.session_persistence is not None:
            args['session_persistence'] = json.loads(
                parsed_args.session_persistence)

        client = self.app.client_manager.network

        obj = client.create_pool(**args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class SetPool(command.ShowOne):
    _description = _('Update a pool')

    def get_parser(self, prog_name):
        parser = super(SetPool, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            help=_('The ID of the pool to update.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Human-readable name of the resource.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('A human-readable description for the resource.')
        )
        parser.add_argument(
            '--session_persistence',
            metavar='<session_persistence>',
            help=_('A JSON object specifying the session persistence '
                   'for the pool or null for no session persistence. '
                   'See Pool Session Persistence. Default is null.')
        )
        parser.add_argument(
            '--lb_algorithm',
            metavar='{' + ','.join(LB_ALGORITHM_VALUES) + '}',
            type=lambda s: s.upper(),
            choices=LB_ALGORITHM_VALUES,
            help=_('The load balancing algorithm for the pool. '
                   'One of [`LEAST_CONNECTIONS`, `ROUND_ROBIN`, `SOURCE_IP`].')
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help=_('Enable pool (default).')
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help=_('Disable pool.')
        )
        return parser

    def take_action(self, parsed_args):

        args = {}

        if parsed_args.lb_algorithm:
            args['lb_algorithm'] = parsed_args.lb_algorithm
        if parsed_args.disable is not None:
            args['is_admin_state_up'] = False
        if parsed_args.name is not None:
            args['name'] = parsed_args.name
        if parsed_args.description is not None:
            args['description'] = parsed_args.description
        if parsed_args.session_persistence is not None:
            args['session_persistence'] = json.loads(
                parsed_args.session_persistence)

        client = self.app.client_manager.network

        obj = client.update_pool(
            pool=parsed_args.pool,
            **args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeletePool(command.Command):
    _description = _('Delete a pool')

    def get_parser(self, prog_name):
        parser = super(DeletePool, self).get_parser(prog_name)

        parser.add_argument(
            'pool',
            metavar='<pool>',
            nargs='+',
            help=_('The ID of the pool to delete.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.network

        for pool in parsed_args.pool:
            obj = client.find_pool(name_or_id=pool, ignore_missing=False)
            client.delete_pool(pool=obj.id, ignore_missing=False)

        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.cbr.v3 import checkpoint as _checkpoint


EXAMPLE = {
    'status': 'protecting',
    'created_at': '2021-02-09T12:03:42.554535',
    'project_id': uuid.uuid4(),
    'id': uuid.uuid4(),
    'vault': {
        'skipped_resources': [],
        'id': uuid.uuid4(),
        'resources': [
            {
                'name': uuid.uuid4(),
                'backup_size': '0',
                'resource_size': '6',
                'protect_status': 'available',
                'backup_count': '0',
                'type': 'OS::Nova::Server',
                'id': uuid.uuid4(),
                'extra_info': {}
            }
        ],
        'name': uuid.uuid4(),
    },
    'extra_info': {
        'retention_duration': -1,
        'name': uuid.uuid4(),
        'description': uuid.uuid4(),
    }
}


class TestCheckpoint(base.TestCase):

    def setUp(self):
        super(TestCheckpoint, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _checkpoint.Checkpoint()
        self.sot_expected = _checkpoint.Checkpoint(**EXAMPLE)

    def test_basic(self):
        sot = _checkpoint.Checkpoint()
        self.assertEqual('checkpoint', sot.resource_key)
        self.assertEqual('', sot.resources_key)
        self.assertEqual('/checkpoints',
                         sot.base_path)
        self.assertFalse(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        test_checkpoint = _checkpoint.Checkpoint(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['status'],
            test_checkpoint.status)
        self.assertEqual(
            EXAMPLE['created_at'],
            test_checkpoint.created_at)
        self.assertEqual(
            EXAMPLE['project_id'],
            test_checkpoint.project_id)
        self.assertEqual(
            EXAMPLE['id'],
            test_checkpoint.id)
        self.assertEqual(
            EXAMPLE['vault']['skipped_resources'],
            test_checkpoint.vault.skipped_resources)
        self.assertEqual(
            EXAMPLE['vault']['id'],
            test_checkpoint.vault.id)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['name'],
            test_checkpoint.vault.resources[0].name)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['backup_size'],
            test_checkpoint.vault.resources[0].backup_size)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['resource_size'],
            test_checkpoint.vault.resources[0].resource_size)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['protect_status'],
            test_checkpoint.vault.resources[0].protect_status)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['backup_count'],
            test_checkpoint.vault.resources[0].backup_count)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['type'],
            test_checkpoint.vault.resources[0].type)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['id'],
            test_checkpoint.vault.resources[0].id)
        self.assertEqual(
            EXAMPLE['vault']['resources'][0]['extra_info'],
            test_checkpoint.vault.resources[0].extra_info)
        self.assertEqual(
            EXAMPLE['vault']['name'],
            test_checkpoint.vault.name)
        self.assertEqual(
            EXAMPLE['extra_info']['retention_duration'],
            test_checkpoint.extra_info.retention_duration)
        self.assertEqual(
            EXAMPLE['extra_info']['name'],
            test_checkpoint.extra_info.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
import uuid

from otcextensions.tests.functional.sdk.dns import TestDns

_logger = openstack._log.setup_logging('openstack')


class TestNameservers(TestDns):
    uuid_v4 = uuid.uuid4().hex[:8]
    zone_alias = uuid_v4 + 'dns.sdk-test-zone-public.com.'
    zones = []

    def setUp(self):
        super(TestNameservers, self).setUp()
        # create zone
        try:
            self.zone = self.client.create_zone(
                name=self.zone_alias
            )
        except openstack.exceptions.BadRequestException:
            self.zone = self.client.find_zone(self.zone_alias)
        self.zones.append(self.zone)

    def tearDown(self):
        try:
            for zone in self.zones:
                if zone:
                    self.client.delete_zone(zone)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestNameservers, self).tearDown()

    def test_list_nameservers(self):
        nameservers = []
        zone = self.client.nameservers(zone=self.zone.id)
        self.assertIsNotNone(zone)
        for ns in zone:
            nameservers.append(ns['hostname'])
        self.assertEqual(nameservers,
                         ['ns1.open-telekom-cloud.com.',
                          'ns2.open-telekom-cloud.com.'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.ces.v1 import alarm


EXAMPLE = {
    'alarm_name': 'alarm-qht9',
    'alarm_action_enabled': False,
    'alarm_description': 'Test description',
    'alarm_enabled': True,
    'metric': {
        'namespace': 'SYS.ECS',
        'metric_name': 'cpu_util',
        'dimensions': [
            {
                'name': 'instance_id',
                'value': 'ccb27344-1ec4-423e-a4d9-f3a885a23e72'
            }
        ]
    },
    'condition': {
        'period': 1,
        'filter': 'average',
        'comparison_operator': '>=',
        'value': 80,
        'unit': '%',
        'count': 3
    }
}


class TestAlarm(base.TestCase):

    def test_basic(self):
        sot = alarm.Alarm()
        self.assertEqual('metric_alarms', sot.resources_key)
        path = '/alarms'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = alarm.Alarm(**EXAMPLE)
        self.assertEqual(EXAMPLE['alarm_name'], sot.name)
        self.assertEqual(EXAMPLE['alarm_action_enabled'],
                         sot.alarm_action_enabled)
        self.assertEqual(EXAMPLE['alarm_description'],
                         sot.alarm_description)
        self.assertEqual(EXAMPLE['alarm_enabled'], sot.alarm_enabled)
        self.assertEqual(EXAMPLE['metric']['namespace'], sot.metric.namespace)
        self.assertEqual(EXAMPLE['metric']['metric_name'],
                         sot.metric.metric_name)
        self.assertEqual(EXAMPLE['metric']['dimensions'][0]['name'],
                         sot.metric.dimensions[0].name)
        self.assertEqual(EXAMPLE['metric']['dimensions'][0]['value'],
                         sot.metric.dimensions[0].value)
        self.assertEqual(EXAMPLE['condition']['period'],
                         sot.condition.period)
        self.assertEqual(EXAMPLE['condition']['filter'],
                         sot.condition.filter)
        self.assertEqual(EXAMPLE['condition']['comparison_operator'],
                         sot.condition.comparison_operator)
        self.assertEqual(EXAMPLE['condition']['value'], sot.condition.value)
        self.assertEqual(EXAMPLE['condition']['unit'], sot.condition.unit)
        self.assertEqual(EXAMPLE['condition']['count'], sot.condition.count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import base64
import hashlib
from collections import namedtuple

import mock
from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.obs.v1 import obj

EXAMPLE = {
    "Key": "name",
    "LastModified": "2013-01-15T05:52:15.920Z<",
    "ETag": "etag",
}

EXAMPLE_LIST = """<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<ListBucketResult xmlns="http://obs.otc.t-systems.com/doc/2016-01-01/">
<Name>ag-test-v1</Name><Prefix></Prefix><Marker></Marker>
<MaxKeys>1000</MaxKeys><IsTruncated>false</IsTruncated>
<Contents><Key>setup.py</Key>
<LastModified>2018-10-02T10:17:04.666Z</LastModified>
<ETag>"9c24605289b49ad77a51ba7986425158"</ETag>
<Size>1030</Size><Owner><ID>859d69896ff44ba6a20845edb43f311e</ID>
<DisplayName>OTC00000000001000000448</DisplayName></Owner>
<StorageClass>STANDARD</StorageClass></Contents></ListBucketResult>
"""

INITIATE_MPU_RESP = '''
<InitiateMultipartUpload>
<UploadId>ID</UploadId>
</InitiateMultipartUpload>
'''

LIST_PARTS_RESP = '''
<ListPartsResult>
<StorageClass>STANDARD</StorageClass>
<PartNumberMarker>1</PartNumberMarker>
<NextPartNumberMarker>3</NextPartNumberMarker>
<MaxParts>2</MaxParts>
<IsTruncated>true</IsTruncated>
<Part>
<PartNumber>2</PartNumber>
<LastModified>2010-11-10T20:48:34.000Z</LastModified>
<ETag>"7778aef83f66abc1fa1e8477f296d394"</ETag>
<Size>10485760</Size>
</Part>
<Part>
<PartNumber>3</PartNumber>
<LastModified>2010-11-10T20:48:33.000Z</LastModified>
<ETag>"aaaa18db4cc2f85cedef654fccc4a4x8"</ETag>
<Size>10485760</Size>
</Part>
</ListPartsResult>
'''

COMPLETE_MPU_RESP = \
    '<CompleteMultipartUpload>' \
    '<Part><PartNumber>1</PartNumber><ETag>1</ETag>' \
    '</Part><Part><PartNumber>2</PartNumber><ETag>2</ETag>' \
    '</Part><Part><PartNumber>3</PartNumber>' \
    '<ETag>3</ETag></Part>' \
    '</CompleteMultipartUpload>'


class TestObject(base.TestCase):

    def setUp(self):
        super(TestObject, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = obj.Object()

        self.assertEqual('/', sot.base_path)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_head)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = obj.Object(**EXAMPLE)
        self.assertEqual(EXAMPLE['Key'], sot.id)
        self.assertEqual(EXAMPLE['Key'], sot.name)
        self.assertEqual(EXAMPLE['LastModified'], sot.last_modified)
        self.assertEqual(EXAMPLE['ETag'], sot.etag)

    def test_list(self):
        sot = obj.Object()

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.content = EXAMPLE_LIST

        self.sess.get.return_value = mock_response

        result = list(sot.list(
            self.sess,
        ))

        self.assertEqual(1, len(result))
        self.assertEqual('setup.py', result[0].name)
        self.assertEqual('9c24605289b49ad77a51ba7986425158', result[0].etag)
        self.assertEqual(1030, result[0].content_length)

    def test_create(self):
        data = 'some test data'
        md5 = hashlib.md5()
        md5.update(str.encode(data))
        data_md5 = base64.b64encode(md5.digest()).decode()
        sot = obj.Object(name='test-v1', data=data)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.content = ''
        mock_response.headers = {}

        self.sess.put.return_value = mock_response

        sot.create(
            self.sess,
            endpoint_override='epo',
            requests_auth=2)

        self.sess.put.assert_called_once_with(
            '/test-v1',
            data=data,
            endpoint_override='epo',
            requests_auth=2,
            headers={
                'Content-MD5': data_md5
            }
        )

    def test_initiate_multipart_upload(self):
        sot = obj.Object()
        return_data = namedtuple('response', ['content'])

        response_data = return_data(INITIATE_MPU_RESP)

        self.sess.post = mock.Mock(return_value=response_data)
        sot.initiate_multipart_upload(
            self.sess,
            'http://obs.otc.t-systems.com',
            'test'
        )

        self.sess.post.assert_called_once_with(
            url='/test?uploads',
            endpoint_override='http://obs.otc.t-systems.com'
        )

    def test_get_parts(self):
        sot = obj.Object()
        return_data = namedtuple('response', ['content'])

        response_data = return_data(LIST_PARTS_RESP)

        self.sess.get = mock.Mock(return_value=response_data)
        sot.get_parts(
            self.sess,
            'http://obs.otc.t-systems.com',
            'test'
        )

        self.sess.get.assert_called_once_with(
            'http://obs.otc.t-systems.com',
            requests_auth='test'
        )

    def test_complete_multipart_upload(self):
        sot = obj.Object()
        input_data = [
            {'PartNumber': '1', 'ETag': '1'},
            {'PartNumber': '2', 'ETag': '2'},
            {'PartNumber': '3', 'ETag': '3'}
        ]

        self.sess.post = mock.Mock(return_value=input_data)
        sot.complete_multipart_upload(
            self.sess,
            'http://obs.otc.t-systems.com',
            'test',
            input_data,
            {},
            requests_auth='requests_auth'
        )

        self.sess.post.assert_called_once_with(
            'http://obs.otc.t-systems.com?uploadId=test',
            data=COMPLETE_MPU_RESP,
            headers={},
            requests_auth='requests_auth'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

import openstack
from openstack import _log

from otcextensions.tests.functional.sdk.mrs import TestMrs

_logger = _log.setup_logging('openstack')


class TestJob(TestMrs):
    job = None

    def setUp(self):
        super(TestJob, self).setUp()
        res = self.client.create_job(
            name=uuid.uuid4().hex,
            type='DistCp',
            description='test job',
        )
        id = res.id
        self.job = self.client.get_job(id)

        _logger.debug("SAHARA create test job")
        _logger.debug(self.job)

    def tearDown(self):
        try:
            pass
            if self.job.id:
                pass
                self.client.delete_job(self.job)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestJob, self).tearDown()

    def test_list(self):
        self.jobs = list(self.client.jobs())
        self.assertGreaterEqual(len(self.jobs), 0)
        for j in self.jobs:
            _logger.debug(j)

    def test_update(self):
        res = self.client.update_job(
            self.job,
            type='DistCp',
            description='funct_test update mapreduce'
        )
        self.assertEqual('funct_test update mapreduce', res.description)
        _logger.debug(res)
        self.jobs = list(self.client.jobs())
        self.assertGreaterEqual(len(self.jobs), 0)
        for j in self.jobs:
            _logger.debug(j)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class LocalNameSpec(resource.Resource):
    en_us = resource.Body('en_us')
    zh_cn = resource.Body('zh_cn')


class AZ(resource.Resource):
    resource_name = 'AZ'
    resources_key = 'available_zones'
    base_path = '/apigw/available-zones'

    allow_list = True

    name = resource.Body('name')
    id = resource.Body('id')
    code = resource.Body('code')
    port = resource.Body('port')
    local_name = resource.Body('local_name', type=LocalNameSpec)
    specs = resource.Body('specs', type=dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource
from openstack import utils


class CredentialSpec(resource.Resource):
    #: OAuth token of GitHub.
    access_token = resource.Body("access_token")
    #: SSH private certificate.
    ssh_private_key = resource.Body("ssh_private_key")


class ConnectionInfoSpec(resource.Resource):
    #: Certificate information.
    credential = resource.Body("credential", type=CredentialSpec)
    #: Repository link protocol.
    protocol = resource.Body("protocol")
    #: Repository link address.
    url = resource.Body("url")


class RepositorySpec(resource.Resource):
    #: Repository branch.
    branch = resource.Body("branch")
    #: Repository link information.
    connection_info = resource.Body("connection_info", type=ConnectionInfoSpec)
    #: Repository ID.
    id = resource.Body("id")
    #: Repository type.
    type = resource.Body("type")
    #: Repository user mailbox.
    user_email = resource.Body("user_email")
    #: Repository username.
    user_name = resource.Body("user_name")


class AutoStopSpec(resource.Resource):
    #: Running duration, in seconds.
    duration = resource.Body("duration", type=int)
    #: Whether to enable the auto stop function.
    enable = resource.Body("enable", type=bool)
    #: Whether to display a prompt again.
    prompt = resource.Body("prompt", type=bool)
    #: Remaining time before actual stop, in seconds.
    remain_time = resource.Body("remain_time", type=int)
    #: Time when the instance stops.
    stop_timestamp = resource.Body("stop_timestamp", type=int)


class FailedReasonsSpec(resource.Resource):
    #: Error code.
    code = resource.Body("code")
    #: Error details.
    detail = resource.Body("detail", type=dict)
    #: Error message.
    message = resource.Body("message")


class LocationSpec(resource.Resource):
    #: Storage pathIf type is set to obs, this parameter is mandatory.
    path = resource.Body("path")
    #: If type is set to obs, this parameter does not need to be set.
    volume_size = resource.Body("volume_size", type=int)
    #: Volume stotage unit.
    volume_unit = resource.Body("volume_unit")


class StorageSpec(resource.Resource):
    #: Storage location.
    location = resource.Body("location", type=LocationSpec)
    #: Storage type.
    type = resource.Body("type")


class NotebookSpec(resource.Resource):
    #: AnnotationsThe generated URL cannot be directly accessed.
    annotations = resource.Body("annotations", type=dict)
    #: Auto stop parameter.
    auto_stop = resource.Body("auto_stop", type=AutoStopSpec)
    #: AK and SK for accessing OBS.
    credential = resource.Body("credential", type=CredentialSpec)
    #: Path for storing custom initialization scripts used when a notebook
    #:  instance is started.
    custom_script_path = resource.Body("custom_script_path")
    #: Extended parameter.
    extend_params = resource.Body("extend_params", type=dict)
    #: Extended storage list.
    extend_storage = resource.Body(
        "extend_storage", type=list, list_type=StorageSpec
    )
    #: Cause for a creation or startup failure.
    failed_reasons = resource.Body("failed_reasons", type=FailedReasonsSpec)
    #: Path for storing custom image logs.
    log_path = resource.Body("log_path")
    #: Git repository information.
    repository = resource.Body("repository", type=RepositorySpec)
    #: Time when the resource is reserved.
    resource_reserved_timestamp = resource.Body(
        "resource_reserved_timestamp", type=int
    )
    #: Storage path.
    storage = resource.Body("storage", type=StorageSpec)


class PoolSpec(resource.Resource):
    #: ID of a resource pool.
    id = resource.Body("id")
    #: Name of a resource pool.
    name = resource.Body("name")
    #: This parameter is mandatory when type is set to USER_DEFINED.
    owner = resource.Body("owner", type=dict)
    #: Type of a resource pool.
    type = resource.Body("type")


class FlavorDetailsSpec(resource.Resource):
    #: Billing specifications.
    billing_flavor = resource.Body("billing_flavor")
    #: Billing ratio This parameter is mandatory when billing_flavor is
    #:  specified.
    billing_params = resource.Body("billing_params", type=int)
    #: Auto stop time after startup, in seconds.
    duration = resource.Body("duration", type=int)
    #: Number of instances of this flavor the current created.
    instance_num = resource.Body("instance_num", type=int)
    #: Whether the current user has the permission to use this flavor.
    is_permitted = resource.Body("is_permitted", type=bool)
    #: Flavor name.
    name = resource.Body("name")
    #: Parameters that describing flavor.
    params = resource.Body("params", type=dict)
    #: Parameters that describing flavor.
    params_extends = resource.Body("params_extends", type=dict)
    #: Promotion type.
    promo_type = resource.Body("promo_type")
    #: Left queuing time, in secondsThis parameter is mandatory when
    #:  promo_type is set to Free and status is set to soldOut.
    queue_left_time = resource.Body("queue_left_time", type=int)
    #: This parameter is mandatory when promo_type is set to Free and status
    #:  is set to soldOut.
    queuing_num = resource.Body("queuing_num", type=int)
    #: Flavor sale status The options are as follows:onSalesoldOut.
    status = resource.Body("status")
    #: Supported storage type.
    storage_list = resource.Body("storage_list", type=list)
    #: Maximum retention period of an inactive instance of this flavor in the
    #:  database, in hoursThe default value is -1, indicating that the instance
    #:  can be permanently saved.
    store_time = resource.Body("store_time", type=int)
    #: Flavor status.
    type = resource.Body("type")


class QueuingInfoSpec(resource.Resource):
    #: Time when an instance starts queuing.
    begin_timestamp = resource.Body("begin_timestamp", type=int)
    #: Development environment type.
    dev_env_type = resource.Body("de_type")
    #: Time when an instance completes queuing.
    end_timestamp = resource.Body("end_timestamp", type=int)
    #: Instance flavor.
    flavor = resource.Body("flavor")
    #: Flavor details, which display the flavor information and whether the
    #:  flavor is sold out.
    flavor_details = resource.Body("flavor_details", type=FlavorDetailsSpec)
    #: Instance ID.
    id = resource.Body("id")
    #: Instance name.
    name = resource.Body("name")
    #: Ranking of an instance in a queue.
    rank = resource.Body("rank", type=int)
    #: Left queuing time, in seconds.
    remain_time = resource.Body("remain_time", type=int)
    #: Instance status.
    status = resource.Body("status")


class DockerSpec(resource.Resource):
    #: Label information, which can be extended.
    annotations = resource.Body("annotations", type=dict)
    #: Image name.
    image_name = resource.Body("image_name")
    #: Image tag.
    image_tag = resource.Body("image_tag")
    #: SWR organization name, which is globally unique.
    namespace = resource.Body("namespace")


class ProvisionSpecSpec(resource.Resource):
    #: Deployment engine.
    engine = resource.Body("engine")
    #: Deployment parameters.
    params = resource.Body("params", type=DockerSpec)


class ProvisionSpec(resource.Resource):
    #: Provision annotations.
    annotations = resource.Body("annotations", type=dict)
    #: Deployment details.
    spec = resource.Body("spec", type=ProvisionSpecSpec)
    #: Deployment type.
    type = resource.Body("type")


class ProfileSpec(resource.Resource):
    #: Development environment type.
    dev_env_type = resource.Body("de_type")
    #: Configuration description.
    description = resource.Body("description")
    #: Hardware, which can be CPU, GPU.
    flavor_type = resource.Body("flavor_type")
    #: Configuration ID.
    id = resource.Body("id")
    #: Label.
    labels = resource.Body("labels", type=dict)
    #: Configuration name.
    name = resource.Body("name")
    #: Deployment information.
    provision = resource.Body("provision", type=ProvisionSpec)


class Devenv(resource.Resource):
    base_path = "/demanager/instances"

    resources_key = "instances"

    _query_mapping = resource.QueryParameters(
        "ai_project",
        "dev_env_type",
        "limit",
        "order",
        "pool_id",
        "provision_type",
        "offset",
        "show_self",
        "sort_by",
        "status",
        "workspace_id",
        dev_env_type="de_type",
        ai_project="ai_project_id",
        sort_by="sortby",
    )

    allow_create = True
    allow_list = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True
    allow_patch = True

    # Properties

    #: AI project.
    ai_project = resource.Body("ai_project", type=dict)
    #: AI project ID.
    ai_project_id = resource.Body("ai_project_id")
    #: Auto stop parameters.
    auto_stop = resource.Body("auto_stop", type=dict)
    #: Time when an instance is created.
    created_at = resource.Body("creation_timestamp")
    #: Current status of an instance.
    current_status = resource.Body("current_status", type=str)
    #: Instance description.
    description = resource.Body("description")
    #: Error code.
    error_code = resource.Body("error_code")
    #: Instance flavor.
    flavor = resource.Body("flavor")
    #: For details about the flavor, see Table 16.
    flavor_details = resource.Body("flavor_details", type=FlavorDetailsSpec)
    #: Instance ID.
    id = resource.Body("id")
    #: Instance name.
    name = resource.Body("name")
    #: For details about the dedicated resource pool, see Table 17.
    pool = resource.Body("pool", type=PoolSpec)
    #: Previous status of an instance.
    previous_state = resource.Body("previous_state")
    #: Configuration information.
    profile = resource.Body("profile", type=ProfileSpec)
    #: Configuration ID.
    profile_id = resource.Body("profile_id")
    #: Queuing information.
    queuing_info = resource.Body("queuing_info", type=QueuingInfoSpec)
    #: Git repository information.
    repository = resource.Body("repository", type=RepositorySpec)
    #: Instance definition For details about parameters of a notebook
    #:  instance, see Table 19.
    spec = resource.Body("spec", type=NotebookSpec)
    #: Instance status.
    status = resource.Body("status")
    #: Latest Update Timestamp.
    updated_at = resource.Body("latest_update_timestamp", type=str)
    #: User information.
    user = resource.Body("user", type=dict)
    #: Workspace.
    workspace = resource.Body("workspace", type=dict)
    #: Workspace ID.
    workspace_id = resource.Body("workspace_id")

    def _action(self, session, action):
        """Perform actions given the message body."""
        url = utils.urljoin(self.base_path, self.id, "action")
        body = {"action": action}
        headers = {
            "Accept": "application/json",
            "Content-type": "application/json",
        }
        response = session.post(url, json=body, headers=headers)
        self._translate_response(response)
        return self

    def start(self, session):
        """start the DevEnv Instance."""
        return self._action(session, "start")

    def stop(self, session):
        """stop the DevEnv Instance."""
        return self._action(session, "stop")

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        exceptions.raise_from_response(response, error_message=error_message)
        if has_body:
            body = response.json()
            if self.resource_key and self.resource_key in body:
                body = body[self.resource_key]

            if body.get("workspace") and body["workspace"].get("id"):
                body["workspace_id"] = body["workspace"]["id"]

            if body.get("ai_project") and body["ai_project"].get("id"):
                body["ai_project_id"] = body["ai_project"]["id"]

            body = self._consume_body_attrs(body)
            self._body.attributes.update(body)
            self._body.clean()
            if self.commit_jsonpatch:
                # We need the original body to compare against
                self._original_body = body.copy()

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import load_balancer

EXAMPLE = {
    'availability_zone_list': ['eu-nl-01'],
    'created_at': 'created_at',
    'description': 'description',
    'deletion_protection_enable': True,
    'eips': [{
        'eip_id': 'eip_id',
        'eip_address': 'eip_address',
        'ip_version': 'ip_version'
    }],
    'publicips': [{
        'eip_id': 'eip_id',
        'eip_address': 'eip_address',
        'ip_version': 'ip_version'
    }],
    'publicip': [{
        'network_type': '5_bgp',
        'billing_info': 'billing-info',
        'description': 'description',
        'ip_version': 4,
        'bandwidth': {
            'size': 2,
            'share_type': 'share-type',
            'charge_mode': 'charge-mode',
            'name': 'name',
            'billing_info': 'billing-info',
            'id': 'id'
        }
    }],
    'guaranteed': True,
    'ip_target_enable': True,
    'l4_flavor_id': 'l4_flavor_id',
    'l7_flavor_id': 'l7_flavor_id',
    'listeners': [{'id': 'lstnr_id_1'}, {'id': 'lstnr_id_2'}],
    'elb_virsubnet_ids': ['uuid1', 'uuid2'],
    'elb_virsubnet_type': 'subnet_type',
    'operating_status': 'operating_status',
    'pools': [{'id': 'pool_id_1'}, {'id': 'pool_id_2'}],
    'project_id': 'project_id',
    'provider': 'provider',
    'provisioning_status': 'provisioning_status',
    'tags': [{
        "key": "key1",
        "value": "value1"
    }],
    'updated_at': 'updated_at',
    'vip_address': 'vip_address',
    'vip_port_id': 'vip_port_id',
    'vip_subnet_cidr_id': 'vip_subnet_id',
    'vpc_id': 'vpc_id',
}


class TestLoadBalancer(base.TestCase):

    def test_basic(self):
        sot = load_balancer.LoadBalancer()
        path = '/elb/loadbalancers'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = load_balancer.LoadBalancer(**EXAMPLE)
        self.assertEqual(EXAMPLE['availability_zone_list'],
                         sot.availability_zones)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['deletion_protection_enable'],
                         sot.deletion_protection_enable)
        self.assertEqual(EXAMPLE['eips'], sot.eips)
        self.assertEqual(EXAMPLE['publicips'], sot.floating_ips)
        self.assertEqual(EXAMPLE['publicip'], sot.floating_ip)
        self.assertEqual(EXAMPLE['guaranteed'], sot.is_guaranteed)
        self.assertEqual(EXAMPLE['ip_target_enable'], sot.ip_target_enable)
        self.assertEqual(EXAMPLE['l4_flavor_id'], sot.l4_flavor_id)
        self.assertEqual(EXAMPLE['l7_flavor_id'], sot.l7_flavor_id)
        self.assertEqual(EXAMPLE['listeners'], sot.listeners)
        self.assertEqual(EXAMPLE['elb_virsubnet_ids'], sot.network_ids)
        self.assertEqual(EXAMPLE['elb_virsubnet_type'], sot.subnet_type)
        self.assertEqual(EXAMPLE['operating_status'], sot.operating_status)
        self.assertEqual(EXAMPLE['pools'], sot.pools)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['provider'], sot.provider)
        self.assertEqual(EXAMPLE['provisioning_status'],
                         sot.provisioning_status)
        self.assertEqual(EXAMPLE['tags'], sot.tags)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)

        self.assertEqual(EXAMPLE['vip_address'], sot.ip_address)
        self.assertEqual(EXAMPLE['vip_port_id'], sot.port_id)
        self.assertEqual(EXAMPLE['vip_subnet_cidr_id'], sot.subnet_id)
        self.assertEqual(EXAMPLE['vpc_id'], sot.vpc_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class ClusterSpec(resource.Resource):
    # Properties
    name = resource.Body('name')
    #: Cluster information.
    cluster = resource.Body('cluster', type=dict)


class ClusterCertificate(resource.Resource):
    base_path = '/clusters/%(cluster_id)s/clustercert'

    allow_fetch = True

    # Properties
    cluster_id = resource.URI('cluster_id')

    #: Certificate authority data.
    ca = resource.Body('ca')
    #: Client certificate.
    client_certificate = resource.Body('client_certificate')
    #: Client key data, containing the PEM data of the TLS key
    #: file of the client.
    client_key = resource.Body('client_key')
    #: Context information.
    context = resource.Body('context', type=dict)

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        exceptions.raise_from_response(response, error_message=error_message)
        if has_body:
            try:
                body = response.json()
                cluster = body['clusters'][0]['cluster']
                self.ca = cluster['certificate-authority-data']
                user = body['users'][0]['user']
                self.client_certificate = user['client-certificate-data']
                self.client_key = user['client-key-data']
                context = body['contexts'][0]
                self.context = {
                    'name': context['name'],
                    'cluster': cluster['server'],
                    'user': context['context']['user']
                }
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts training job version v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class TrainingjobVersionLogfileName(command.ShowOne):
    _description = _("Obtain the name of a training job log file")

    def get_parser(self, prog_name):
        parser = super(TrainingjobVersionLogfileName, self).get_parser(
            prog_name
        )
        parser.add_argument(
            "--job_id",
            metavar="<job_id>",
            required=True,
            type=int,
            help=_("ID of a training job"),
        )
        parser.add_argument(
            "--version_id",
            metavar="<version_id>",
            required=True,
            type=int,
            help=_("Version ID of a training job"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        attrs = {}
        args_list = ["job_id", "version_id"]
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.show_trainingjob_version_logfile_name(**attrs)
        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)

        return display_columns, data



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class AlertConfig(resource.Resource):

    base_path = '/warnalert/alertconfig/query'

    # capabilities
    allow_fetch = True

    # Properties
    #: warn alert config information
    #: *Type: dict*
    config = resource.Body('warn_config', type=dict)
    #: unique topic id
    topic_urn = resource.Body('topic_urn')
    #: warn alert group name
    display_name = resource.Body('display_name')

    # # This overrides the default behavior of resource get
    # def get(self, session, requires_id=False):
    #     return super(AlertConfig, self).get(session, requires_id=requires_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import service_description

from otcextensions.sdk.dms.v1 import _proxy as _proxy_v1


class DmsService(service_description.ServiceDescription):
    """The DMS service."""

    supported_versions = {
        '1': _proxy_v1.Proxy
    }



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts Service Update Logs v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ServiceLogs(command.Lister):
    _description = _("List update logs of a real-time service.")
    columns = (
        "Update Time",
        "Result",
        "Config",
        "Result Detail",
    )

    def get_parser(self, prog_name):
        parser = super(ServiceLogs, self).get_parser(prog_name)

        parser.add_argument(
            "service",
            metavar="<service>",
            help=_("Service ID or Name."),
        )
        parser.add_argument(
            "--update-time",
            metavar="<update_time>",
            type=int,
            help=_(
                "Update time for filtering. This parameter can be used to "
                "obtain the update logs of a real-time service. By default, "
                "the filtering by update time is disabled."
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        query_params = {}
        if parsed_args.update_time:
            query_params["update_time"] = parsed_args.update_time

        service = client.find_service(
            parsed_args.service, ignore_missing=False
        )
        data = client.service_logs(service.id, **query_params)

        formatters = {
            "Update Time": cli_utils.UnixTimestampFormatter,
            "Config": cli_utils.YamlFormat,
        }

        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log

from collections import defaultdict

from otcextensions.sdk import sdk_resource


_logger = _log.setup_logging('openstack')


class BaseResource(sdk_resource.Resource):
    OBS_NS = "http://obs.otc.t-systems.com/doc/2016-01-01/"

    @classmethod
    def etree_to_dict(cls, t):
        """Convert ETree to python dict
        """
        if cls.OBS_NS in t.tag:
            t.tag = t.tag.lstrip('{%s}' % (cls.OBS_NS))
        d = {t.tag: {} if t.attrib else None}
        children = list(t)
        if children:
            dd = defaultdict(list)
            for dc in map(cls.etree_to_dict, children):
                for k, v in dc.items():
                    dd[k].append(v)
            d = {t.tag: {k: v[0] if len(v) == 1 else v for k, v in dd.items()}}
        if t.attrib:
            d[t.tag].update(('@' + k, v) for k, v in t.attrib.items())
        if t.text:
            # strip spaces and quotes
            text = t.text.strip(' "')
            if children or t.attrib:
                if text:
                    d[t.tag]['#text'] = text
            else:
                d[t.tag] = text
        return d



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random

from otcextensions.tests.functional.sdk.dds import TestDds


class TestInstance(TestDds):
    def setUp(self):
        super(TestInstance, self).setUp()

    def test_01_get_policies(self):
        result = self.client.get_policy()
        self.assertIsNotNone(result)

    def test_02_set_policy(self):
        new_number = random.randrange(1, 7)
        attrs = {
            "enabled": True,
            "retention_period_in_days": new_number
        }
        self.client.create_policy(**attrs)
        result = self.client.get_policy()
        self.assertEqual(new_number, result['retention_period_in_days'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

import mock
from keystoneauth1 import adapter

from openstack.tests.unit import base
from otcextensions.sdk.css.v1 import cluster
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal


CLUSTER_ID = uuid.uuid4().hex

EXAMPLE = {
    'actionProgress': {},
    'actions': [],
    'authorityEnable': False,
    'backupAvailable': False,
    'bandwidthSize': 0,
    'created': '2024-04-02T23:23:31',
    'cmk_id': 'cmk-id',
    'datastore': {'type': 'elasticsearch', 'version': '7.10.2'},
    'diskEncrypted': False,
    'elbWhiteList': {'whiteList': '', 'enableWhiteList': False},
    'endpoint': [
        '192.168.0.194:9200',
        '192.168.0.12:9200',
        '192.168.0.66:9200',
    ],
    'enterpriseProjectId': '0',
    'httpsEnable': False,
    'id': CLUSTER_ID,
    'instances': [
        {
            'azCode': 'eu-de-01',
            'id': 'node-id',
            'ip': '192.168.0.194',
            'name': 'test-cluster-ess-esn-1-1',
            'specCode': 'css.xlarge.2',
            'status': '200',
            'type': 'ess',
            'volume': {
                'type': 'COMMON',
                'size': 100,
            },
        },
    ],
    'name': 'test-cluster',
    'period': False,
    'publicIp': None,
    'publicKibanaResp': None,
    'securityGroupId': 'sg-id',
    'status': '200',
    'subnetId': 'network-id',
    'tags': [
        {'key': 'key0', 'value': 'value0'},
        {'key': 'key1', 'value': 'value1'},
    ],
    'updated': '2024-04-03T10:37:44',
    'vpcId': 'router-id',
}


EXAMPLE_CREATE = {
    'name': 'test-cluster',
    'datastore': {'type': 'elasticsearch', 'version': '7.10.2'},
    'instanceNum': 3,
    'httpsEnable': False,
    'diskEncryption': {
        'systemEncrypted': '1',
        'systemCmkid': 'cmk-id',
    },
    'instance': {
        'availability_zone': 'eu-de-01',
        'flavorRef': 'css.xlarge.2',
        'volume': {'volume_type': 'COMMON', 'size': 100},
        'nics': {
            'vpcId': 'vpc-id',
            'netId': 'net-id',
            'securityGroupId': 'sg-id',
        },
    },
    'tags': [
        {'key': 'key0', 'value': 'value0'},
        {'key': 'key1', 'value': 'value1'},
    ],
    'backupStrategy': {
        'period': '00:00 GMT+03:00',
        'prefix': 'backup',
        'keepday': 1,
        'bucket': 'css-test-0',
        'agency': 'test-css',
        'basePath': 'css',
    },
}


class TestCluster(base.TestCase):
    def setUp(self):
        super(TestCluster, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = cluster.Cluster()

        self.assertEqual('/clusters', sot.base_path)
        self.assertEqual('cluster', sot.resource_key)
        self.assertEqual('clusters', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_patch)
        self.assertDictEqual(
            {'start': 'start', 'limit': 'limit', 'marker': 'marker'},
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = cluster.Cluster(**EXAMPLE)
        updated_sot_attrs = {
            'actionProgress': 'action_progress',
            'authorityEnable': 'is_authority_enabled',
            'backupAvailable': 'is_backup_enabled',
            'backupStrategy': 'backup_strategy',
            'bandwidthSize': 'bandwidth_size',
            'created': 'created_at',
            'diskEncrypted': 'is_disk_encrypted',
            'httpsEnable': 'is_https_enabled',
            'period': 'is_billed',
            'publicIp': 'floating_ip',
            'publicKibanaResp': 'public_kibana_resp',
            'securityGroupId': 'security_group_id',
            'subnetId': 'network_id',
            'updated': 'updated_at',
            'vpcId': 'router_id',
            'endpoint': 'endpoints',
            'enterpriseProjectId': 'enterprise_project_id',
            'diskEncryption': 'disk_encryption',
        }

        for key, value in EXAMPLE.items():
            if key in updated_sot_attrs.keys():
                self.assertEqual(getattr(sot, updated_sot_attrs[key]), value)
            elif key == 'instances':
                pass
            elif key == 'elbWhiteList':
                self.assertEqual(
                    sot.elb_whitelist.is_whitelist_enabled,
                    value['enableWhiteList'],
                )
                self.assertEqual(
                    sot.elb_whitelist.whitelist, value['whiteList']
                )
            else:
                assert_attributes_equal(self, getattr(sot, key), value)

        for i in range(len(sot.nodes)):
            instance = EXAMPLE['instances'][i]
            self.assertEqual(
                instance['azCode'], sot.nodes[i].availability_zone
            )
            self.assertEqual(sot.nodes[i].id, instance['id'])
            self.assertEqual(sot.nodes[i].ip, instance['ip'])
            self.assertEqual(sot.nodes[i].name, instance['name'])
            self.assertEqual(sot.nodes[i].flavor, instance['specCode'])
            self.assertEqual(sot.nodes[i].status, instance['status'])
            self.assertEqual(sot.nodes[i].type, instance['type'])
            assert_attributes_equal(
                self, sot.nodes[i].volume, instance['volume']
            )

    def test_create_sot(self):
        updated_sot_attrs = {
            'instanceNum': 'instance_num',
            'httpsEnable': 'is_https_enabled',
        }
        sot = cluster.Cluster(**EXAMPLE_CREATE)
        for key, value in EXAMPLE_CREATE.items():
            if key in updated_sot_attrs.keys():
                self.assertEqual(getattr(sot, updated_sot_attrs[key]), value)
            elif key == 'diskEncryption':
                self.assertEqual(
                    sot.disk_encryption.system_encrypted,
                    value['systemEncrypted'],
                )
                self.assertEqual(
                    sot.disk_encryption.system_cmkid, value['systemCmkid']
                )
            elif key == 'instance':
                self.assertEqual(
                    sot.instance.nics.network_id, value['nics']['netId']
                )
                self.assertEqual(
                    sot.instance.nics.router_id, value['nics']['vpcId']
                )
                self.assertEqual(
                    sot.instance.nics.security_group_id,
                    value['nics']['securityGroupId'],
                )
                self.assertEqual(sot.instance.flavor, value.pop('flavorRef'))
                del value['nics']
                assert_attributes_equal(self, sot.instance, value)
            elif key == 'backupStrategy':
                self.assertEqual(
                    sot.backup_strategy.base_path, value.pop('basePath')
                )
                assert_attributes_equal(self, sot.backup_strategy, value)
            else:
                assert_attributes_equal(self, getattr(sot, key), value)

    def test_action(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        action = 'restart'
        json_body = {'restart': {}}
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.post.return_value = response

        rt = sot._action(self.sess, action, json_body)
        self.sess.post.assert_called_with(
            'clusters/%s/restart' % sot.id, json=json_body
        )

        self.assertIsNone(rt)

    def test_restart(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()

        rt = sot.restart(self.sess)
        sot._action.assert_called_with(self.sess, 'restart')
        self.assertIsNone(rt)

    def test_extend(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()
        node_count = 3

        rt = sot.extend(self.sess, node_count)
        sot._action.assert_called_with(
            self.sess, 'extend', {'grow': {'modifySize': node_count}}
        )
        self.assertIsNone(rt)

    def test_update_name(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()
        new_name = 'test_update_name'

        body = {'displayName': new_name}

        rt = sot.update_name(self.sess, new_name)
        sot._action.assert_called_with(self.sess, 'changename', body)
        self.assertIsNone(rt)

    def test_update_password(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()
        new_password = 'test_new_123_Pass'

        body = {'newpassword': new_password}

        rt = sot.update_password(self.sess, new_password)
        sot._action.assert_called_with(self.sess, 'password/reset', body)
        self.assertIsNone(rt)

    def test_update_security_mode(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()

        authority_enable = True
        https_enable = True
        admin_pwd = None

        body = {
            'authorityEnable': authority_enable,
            'httpsEnable': https_enable,
        }

        rt1 = sot.update_security_mode(
            self.sess, https_enable, authority_enable, admin_pwd
        )
        sot._action.assert_called_with(self.sess, 'mode/change', body)
        self.assertIsNone(rt1)

        admin_pwd = 'new_Pass_123'
        body['adminPwd'] = admin_pwd

        rt2 = sot.update_security_mode(
            self.sess, https_enable, authority_enable, admin_pwd
        )
        sot._action.assert_called_with(self.sess, 'mode/change', body)
        self.assertIsNone(rt2)

    def test_update_security_group(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()
        security_group_id = 'test_security_group_id'

        body = {'security_group_ids': security_group_id}

        rt = sot.update_security_group(self.sess, security_group_id)
        sot._action.assert_called_with(self.sess, 'sg/change', body)
        self.assertIsNone(rt)

    def test_update_kernel(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()

        target_image_id = 'target_image_id'
        upgrade_type = 'same'
        indices_backup_check = True
        agency = 'test-agency'

        body = {
            'target_image_id': target_image_id,
            'upgrade_type': upgrade_type,
            'indices_backup_check': indices_backup_check,
            'agency': agency,
            'cluster_load_check': True,
        }

        rt1 = sot.update_kernel(
            self.sess,
            target_image_id,
            upgrade_type,
            indices_backup_check,
            agency,
        )
        sot._action.assert_called_with(
            self.sess, 'inst-type/all/image/upgrade', body
        )
        self.assertIsNone(rt1)

    def test_update_flavor(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()
        new_flavor = 'test_update_name'
        body = {'needCheckReplica': True, 'newFlavorId': new_flavor}

        rt1 = sot.update_flavor(self.sess, new_flavor, check_replica=True)
        sot._action.assert_called_with(self.sess, 'flavor', body)
        self.assertIsNone(rt1)

        rt2 = sot.update_flavor(
            self.sess, new_flavor, node_type='ess', check_replica=True
        )
        sot._action.assert_called_with(self.sess, 'ess/flavor', body)
        self.assertIsNone(rt2)

    def test_scale_in(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()
        nodes = ['5e134b90-8159-4333-9dae-4305029a838a']

        body = {
            'shrinkNodes': nodes,
        }

        rt = sot.scale_in(self.sess, nodes)
        sot._action.assert_called_with(self.sess, 'node/offline', body)
        self.assertIsNone(rt)

    def test_scale_in_by_node_type(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        nodes = [{'type': 'ess', 'reducedNodeNum': 1}]
        json_body = {'shrink': nodes}
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.post.return_value = response

        endpoint = 'https://test-url.com/v1.0/project-id'
        updated_endpoint = 'https://test-url.com/v1.0/extend/project-id'

        self.sess.get_endpoint.return_value = endpoint

        rt = sot.scale_in_by_node_type(self.sess, nodes)
        self.sess.post.assert_called_with(
            f'{updated_endpoint}/clusters/{CLUSTER_ID}/role/shrink',
            json=json_body
        )

        self.assertIsNone(rt)

    def test_replace_node(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        node_id = 'test-id'
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.put.return_value = response

        rt = sot.replace_node(self.sess, node_id)
        self.sess.put.assert_called_with(
            f'clusters/{sot.id}/instance/{node_id}/replace'
        )

        self.assertIsNone(rt)

    def test_add_nodes(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        sot._action = mock.Mock()

        node_type = 'ess-client'
        flavor = 'flavor-id'
        node_size = 3
        volume_type = 'volume-type'
        body = {
            'type': {
                'flavor_ref': flavor,
                'node_size': node_size,
                'volume_type': volume_type,
            }
        }

        rt = sot.add_nodes(
            self.sess, node_type, flavor, node_size, volume_type
        )
        sot._action.assert_called_with(
            self.sess, f'type/{node_type}/independent', body
        )
        self.assertIsNone(rt)

    def test_retry_upgrade_job(self):
        sot = cluster.Cluster.existing(id=CLUSTER_ID)
        job_id = 'action-id'
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.put.return_value = response

        rt1 = sot.retry_upgrade_job(self.sess, job_id)
        self.sess.put.assert_called_with(
            f'clusters/{sot.id}/upgrade/{job_id}/retry', params={}
        )

        self.assertIsNone(rt1)

        retry_mode = 'abort'
        rt2 = sot.retry_upgrade_job(self.sess, job_id, retry_mode)
        self.sess.put.assert_called_with(
            f'clusters/{sot.id}/upgrade/{job_id}/retry',
            params={'retry_mode': retry_mode},
        )

        self.assertIsNone(rt2)


class TestExtendClusterNodes(base.TestCase):
    def setUp(self):
        super(TestExtendClusterNodes, self).setUp()

    def test_basic(self):
        sot = cluster.ExtendClusterNodes()

        self.assertEqual('/clusters/%(cluster_id)s/role_extend', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_patch)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        request = {
            'grow': [
                {'type': 'ess-master', 'nodesize': 2, 'disksize': 0},
                {'type': 'ess', 'nodesize': 0, 'disksize': 60},
            ]
        }
        sot = cluster.ExtendClusterNodes(**request)
        self.assertEqual(request['grow'], sot.grow)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
from openstack import exceptions
from openstack import resource


from otcextensions.tests.functional import base
import uuid
import random

_logger = openstack._log.setup_logging('openstack')


class TestInstance(base.BaseFunctionalTest):
    UUID = uuid.uuid4().hex[:8]
    INSTANCE_NAME = 'sdk-test-dms-' + UUID
    ROUTER_NAME = 'sdk-test-router-' + UUID
    NET_NAME = 'sdk-test-net-' + UUID
    SUBNET_NAME = 'sdk-test-subnet-' + UUID
    SG_NAME = 'sdk-test-sg-' + UUID
    ROUTER_ID = None
    NET_ID = None
    SUBNET_ID = None
    SG_ID = None
    instances = []

    def setUp(self):
        super(TestInstance, self).setUp()
        self._initialize_network()
        products = list(self.conn.dms.products())
        products_ids = [product.product_id for product in products]
        product_id = random.choice(products_ids)
        availability_zone_names = [
            product.availability_zones for product in products
            if product_id == product.product_id
        ]
        availability_zone_name = random.choice(
            sum(availability_zone_names, [])
        )
        availability_zone_id = [
            availability_zone.id
            for availability_zone in self.conn.dms.availability_zones()
            if availability_zone_name == availability_zone.code
        ]
        try:
            self.instance = self.conn.dms.create_instance(
                name=self.INSTANCE_NAME,
                engine="kafka", engine_version="2.3.0", storage=4800,
                router_id=self.ROUTER_ID, network_id=self.NET_ID,
                security_group_id=self.SG_ID,
                availability_zones=availability_zone_id,
                product_id=product_id,
                storage_spec_code="dms.physical.storage.ultra"
            )
        except exceptions.DuplicateResource:
            self.instance = self.conn.dms.find_instance(
                alias=self.INSTANCE_NAME
            )
        resource.wait_for_status(
            session=self.conn.dms,
            resource=self.instance,
            status='RUNNING',
            failures=['CREATEFAILED'],
            interval=5,
            wait=900)
        self.assertIn('instance_id', self.instance)
        self.instances.append(self.instance)

    def tearDown(self):
        try:
            for instance in self.instances:
                if instance.id:
                    self.conn.dms.delete_instance(instance)
                    resource.wait_for_delete(
                        session=self.conn.dms,
                        resource=instance,
                        interval=2,
                        wait=60)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        finally:
            self._deinitialize_network()
            super(TestInstance, self).tearDown()

    def test_list(self):
        self.all_instances = list(self.conn.dms.instances())
        self.assertGreaterEqual(len(self.all_instances), 0)
        if len(self.all_instances) > 0:
            instance = self.all_instances[0]
            instance = self.conn.dms.get_instance(instance=instance.id)
            self.assertIsNotNone(instance)

    def _initialize_network(self):
        self.cidr = '192.168.0.0/16'
        self.ipv4 = 4

        sg = self.conn.network.create_security_group(name=self.SG_NAME)
        self.assertEqual(self.SG_NAME, sg.name)
        TestInstance.SG_ID = sg.id

        network = self.conn.network.create_network(name=self.NET_NAME)
        self.assertEqual(self.NET_NAME, network.name)
        TestInstance.NET_ID = network.id

        subnet = self.conn.network.create_subnet(
            name=self.SUBNET_NAME,
            ip_version=self.ipv4,
            network_id=TestInstance.NET_ID,
            cidr=self.cidr
        )
        self.assertEqual(self.SUBNET_NAME, subnet.name)
        TestInstance.SUBNET_ID = subnet.id

        router = self.conn.network.create_router(name=self.ROUTER_NAME)
        self.assertEqual(self.ROUTER_NAME, router.name)
        TestInstance.ROUTER_ID = router.id
        interface = router.add_interface(self.conn.network,
                                         subnet_id=TestInstance.SUBNET_ID)
        self.assertEqual(interface['subnet_id'], TestInstance.SUBNET_ID)
        self.assertIn('port_id', interface)
        # add CIDR to Router for compatibility
        service = self.conn.identity.find_service('vpc')
        endpoints = list(self.conn.identity.endpoints(service_id=service.id))
        endpoint_url = endpoints[0].url
        endpoint = endpoint_url[:endpoint_url.rfind("/") + 1]
        cidr = self.conn.compute.put(
            endpoint + self.conn.session.get_project_id() + '/vpcs/'
            + self.ROUTER_ID, data='{"vpc": {"cidr": "192.168.0.0/16"}}',
            headers={'content-type': 'application/json'})
        self.assertIn('cidr', cidr.json()['vpc'])
        self.assertIn('192.168.0.0/16', cidr.json()['vpc']['cidr'])

    def _deinitialize_network(self):
        router = self.conn.network.get_router(TestInstance.ROUTER_ID)
        interface = router.remove_interface(self.conn.network,
                                            subnet_id=TestInstance.SUBNET_ID)
        self.assertEqual(interface['subnet_id'], TestInstance.SUBNET_ID)
        self.assertIn('port_id', interface)
        sot = self.conn.network.delete_router(
            TestInstance.ROUTER_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_subnet(
            TestInstance.SUBNET_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_network(
            TestInstance.NET_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_security_group(
            TestInstance.SG_ID, ignore_missing=False)
        self.assertIsNone(sot)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk import sdk_resource


class BackupTask(sdk_resource.Resource):
    """Cloud Backup"""
    resources_key = "tasks"
    base_path = "/backuppolicy/%(policy_id)s/backuptasks"

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        "sort_dir", "sort_key", "status",
        "limit", "marker", "offset", "status",
        id="job_id")

    #: Properties
    #: Task job id
    id = resource.Body("job_id", alternate_id=True)
    #: Name of backup created by this task name
    backup_name = resource.Body("backup_name")
    #: Resource ID (volume-id for example)
    resource_id = resource.Body("resource_id")
    #: Resource Type (volume for example)
    resource_type = resource.Body("resource_type")
    #: Task status, valid values include: ``RUNNING``, ``EXECUTE_TIMEOUT``,
    #: ``WAITING``, EXECUTE_FAIL``, ``EXECUTE_SUCCESS``
    status = resource.Body("status")
    #: task created at
    created_at = resource.Body("created_at")
    #: task finished at
    finished_at = resource.Body("finished_at")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import log


EXAMPLE = {
    'group_name': 'group-xxx',
    'group_id': 'xxx',
    'stream_id': 'id-xxx',
    'stream_name': 'xxx'
}


class TestFunctionInvocation(base.TestCase):

    def test_basic(self):
        sot = log.Log()
        path = '/fgs/functions/%(function_urn)s/lts-log-detail'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = log.Log(**EXAMPLE)
        self.assertEqual(EXAMPLE['group_name'], sot.group_name)
        self.assertEqual(EXAMPLE['group_id'], sot.group_id)
        self.assertEqual(EXAMPLE['stream_id'], sot.stream_id)
        self.assertEqual(EXAMPLE['stream_name'], sot.stream_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class Job(resource.Resource):

    base_path = '/jobs'

    # capabilities
    allow_fetch = True

    # Properties
    #: Created (RDS)
    created = resource.Body('created')
    #: id
    id = resource.Body('job_id', alternate_id=True)
    #: Additional sub information
    entities = resource.Body('entities', type=dict)
    #: Error code
    error_code = resource.Body('error_code')
    #: Fail reason
    fail_reason = resource.Body('fail_reason')
    #: Job finish time
    finish_time = resource.Body('end_time')
    #: Job start time
    start_time = resource.Body('begin_time', alias='created')
    #: Status
    status = resource.Body('status')
    #: Job type
    #: *Type:str*
    type = resource.Body('job_type')

    def fetch(self, session, requires_id=True,
              base_path=None, error_message=None, **params):
        """Get a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param boolean requires_id: A boolean indicating whether resource ID
                                    should be part of the requested URI.
        :param str base_path: Base part of the URI for fetching resources, if
                              different from
                              :data:`~openstack.resource.Resource.base_path`.
        :param str error_message: An Error message to be returned if
                                  requested object does not exist.
        :param dict params: Additional parameters that can be consumed.
        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_fetch` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
                 the resource was not found.
        """
        if not self.allow_fetch:
            raise exceptions.MethodNotSupported(self, "fetch")

        request = self._prepare_request(requires_id=requires_id,
                                        base_path=base_path)
        if session.service_type == 'rdsv3':
            request.url = self.base_path + '?id=' + self.id
            self.resource_key = 'job'
        session = self._get_session(session)
        microversion = self._get_microversion(session)
        response = session.get(request.url, microversion=microversion,
                               params=params)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self.microversion = microversion
        self._translate_response(response, **kwargs)
        return self


class JobProxyMixin(object):

    def wait_for_job(self, job_id, status='success',
                     failures=None, interval=5, wait=3600,
                     attribute='status'):
        if self.service_type == 'rdsv3':
            status = 'completed'
            failures = ['failed']
        else:
            failures = ['FAIL'] if failures is None else failures

        job = Job.existing(id=job_id).fetch(self)
        return resource.wait_for_status(
            self, job, status, failures, interval, wait, attribute='status'
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.cts.v1 import tracker
from otcextensions.tests.unit.osclient.cts.v1 import fakes


class TestShowTracker(fakes.TestCTS):

    _data = fakes.FakeTracker.create_one()

    columns = (
        'bucket_name', 'file_prefix_name', 'id', 'name',
        'smn', 'status')

    data = (
        _data.bucket_name,
        _data.file_prefix_name,
        _data.id,
        _data.name,
        _data.smn,
        _data.status
    )

    def setUp(self):
        super(TestShowTracker, self).setUp()

        self.cmd = tracker.ShowTracker(self.app, None)

        self.client.get_tracker = mock.Mock()

    def test_show_default(self):
        arglist = [
            'name_or_id',
        ]
        verifylist = [
            ('tracker', 'name_or_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_tracker.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_tracker.assert_called_with(
            tracker='name_or_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteTracker(fakes.TestCTS):

    def setUp(self):
        super(TestDeleteTracker, self).setUp()

        self.cmd = tracker.DeleteTracker(self.app, None)

        self.client.delete_tracker = mock.Mock()

    def test_delete_multiple(self):
        arglist = [
            't1',
            't2',
        ]
        verifylist = [
            ('tracker', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_tracker.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(tracker='t1', ignore_missing=False),
            mock.call(tracker='t2', ignore_missing=False)
        ]

        self.client.delete_tracker.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_tracker.call_count)


class TestCreateTracker(fakes.TestCTS):

    _data = fakes.FakeTracker.create_one()

    columns = (
        'bucket_name', 'file_prefix_name', 'id', 'name',
        'smn', 'status')

    data = (
        _data.bucket_name,
        _data.file_prefix_name,
        _data.id,
        _data.name,
        _data.smn,
        _data.status
    )

    def setUp(self):
        super(TestCreateTracker, self).setUp()

        self.cmd = tracker.CreateTracker(self.app, None)

        self.client.create_tracker = mock.Mock()

    def test_create_default(self):
        arglist = [
            '--bucket_name', '1',
            '--file_prefix_name', '2',
            '--enable_smn',
            '--topic_id', '4',
            '--operation', 'create',
            '--operation', 'delete',
            '--send_all_key',
            '--notify_user', '8',
            '--notify_user', '9',
        ]
        verifylist = [
            ('bucket_name', '1'),
            ('file_prefix_name', '2'),
            ('enable_smn', True),
            ('topic_id', '4'),
            ('operation', ['create', 'delete']),
            ('send_all_key', True),
            ('notify_user', ['8', '9']),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_tracker.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_tracker.assert_called_with(
            bucket_name='1',
            file_prefix_name='2',
            smn={
                'enable': True,
                'topic_id': '4',
                'operations': ['create', 'delete'],
                'notify_users': ['8', '9']
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_no_smn(self):
        arglist = [
            '--bucket_name', '1',
            '--file_prefix_name', '2',
            '--topic_id', '4',
            '--operation', 'create',
            '--operation', 'delete',
            '--send_all_key',
            '--notify_user', '8',
            '--notify_user', '9',
        ]
        verifylist = [
            ('bucket_name', '1'),
            ('file_prefix_name', '2'),
            ('enable_smn', False),
            ('topic_id', '4'),
            ('operation', ['create', 'delete']),
            ('send_all_key', True),
            ('notify_user', ['8', '9']),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_tracker.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_tracker.assert_called_with(
            bucket_name='1',
            file_prefix_name='2',
            smn={
                'enable': False,
            }
        )


class TestSetTracker(fakes.TestCTS):

    _data = fakes.FakeTracker.create_one()

    columns = (
        'bucket_name', 'file_prefix_name', 'id', 'name',
        'smn', 'status')

    data = (
        _data.bucket_name,
        _data.file_prefix_name,
        _data.id,
        _data.name,
        _data.smn,
        _data.status
    )

    def setUp(self):
        super(TestSetTracker, self).setUp()

        self.cmd = tracker.SetTracker(self.app, None)

        self.client.update_tracker = mock.Mock()

    def test_update_default(self):
        arglist = [
            'system',
            '--bucket_name', '1',
            '--file_prefix_name', '2',
            '--enable_smn',
            '--topic_id', '4',
            '--operation', 'create',
            '--operation', 'delete',
            '--send_all_key',
            '--notify_user', '8',
            '--notify_user', '9',
            '--enable'
        ]
        verifylist = [
            ('tracker', 'system'),
            ('bucket_name', '1'),
            ('file_prefix_name', '2'),
            ('enable_smn', True),
            ('topic_id', '4'),
            ('operation', ['create', 'delete']),
            ('send_all_key', True),
            ('notify_user', ['8', '9']),
            ('enable', True)
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_tracker.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_tracker.assert_called_with(
            tracker='system',
            bucket_name='1',
            file_prefix_name='2',
            smn={
                'enable': True,
                'topic_id': '4',
                'operations': ['create', 'delete'],
                'notify_users': ['8', '9']
            },
            status='enabled'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_update_disable(self):
        arglist = [
            'system',
            '--bucket_name', '1',
            '--disable'
        ]
        verifylist = [
            ('tracker', 'system'),
            ('bucket_name', '1'),
            # ('enable_smn', False),
            # ('send_all_key', False),
            ('disable', True)
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_tracker.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_tracker.assert_called_with(
            tracker='system',
            bucket_name='1',
            smn={'enable': False},
            status='disabled'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.kms.v1 import _base


class Random(_base.Resource):

    create_path = '/kms/gen-random'

    allow_create = True
    allow_get = False

    # Properties
    #: Random data length
    #: *Type:str*
    random_data_length = resource.Body('random_data_length', type=int)
    #: Random data content

    #: *Type:str*
    random_data = resource.Body('random_data')


class InstanceNumber(_base.Resource):

    base_path = 'kms/user-instances'
    allow_fetch = True
    # Properties
    #: Instance number
    #: *Type: int*
    instance_num = resource.Body('instance_num', type=int)

    def fetch(self, session):
        return super(InstanceNumber, self).fetch(session, requires_id=False)


class Quota(_base.Resource):

    base_path = 'kms/user-quotas'
    # Properties
    # Resource type
    type = resource.Body('type')
    #: Used resource
    #: *Type: int*
    used = resource.Body('used', type=int)
    #: Quota number for this kind of resource
    #: *Type: int*
    quota = resource.Body('quota', type=int)

    @classmethod
    def list(cls, session):
        session = cls._get_session(session)
        url = cls.base_path
        response = session.get(url)
        resp = response.json()
        if 'error' in resp:
            return
        resources = resp['quotas']['resources']
        for r in resources:
            value = cls.existing(**r)
            yield value



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from osc_lib import exceptions

from otcextensions.osclient.auto_scaling.v1 import instance
from otcextensions.tests.unit.osclient.auto_scaling.v1 import fakes


class TestAutoScalingInstance(fakes.TestAutoScaling):

    def setUp(self):
        super(TestAutoScalingInstance, self).setUp()
        self.client = self.app.client_manager.auto_scaling


class TestListAutoScalingInstance(TestAutoScalingInstance):

    instances = fakes.FakeInstance.create_multiple(3)

    columns = ('ID', 'Name', 'scaling_group_name',
               'scaling_configuration_id', 'scaling_configuration_name',
               'lifecycle_state', 'health_status', 'create_time')

    data = []

    for s in instances:
        data.append((
            s.id,
            s.name,
            s.scaling_group_name,
            s.scaling_configuration_id,
            s.scaling_configuration_name,
            s.lifecycle_state,
            s.health_status,
            s.create_time,
        ))

    def setUp(self):
        super(TestListAutoScalingInstance, self).setUp()

        self.cmd = instance.ListAutoScalingInstance(self.app, None)

        self.client.instances = mock.Mock()

    def test_list(self):
        arglist = [
            '--group', 'grp',
            '--life-cycle-state', 'lc',
            '--health-status', 'hs',
            '--limit', '12'
        ]

        verifylist = [
            ('group', 'grp'),
            ('life_cycle_state', 'lc'),
            ('health_status', 'hs'),
            ('limit', 12)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        grp_mock = mock.Mock()
        grp_mock.id = 2

        self.client.find_group = mock.Mock(return_value=grp_mock)
        self.client.groups.side_effect = [
            self.instances
        ]

        self.client.instances.side_effect = [
            self.instances
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.instances.assert_called_once_with(
            group=grp_mock.id,
            life_cycle_state='lc',
            health_status='hs',
            limit=12)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestDeleteAutoScalingInstance(TestAutoScalingInstance):

    def setUp(self):
        super(TestDeleteAutoScalingInstance, self).setUp()

        self.cmd = instance.RemoveAutoScalingInstance(self.app, None)

        self.client.delete_instance = mock.Mock()

    def test_remove(self):
        arglist = [
            'Instance1',
            '--delete-instance'
        ]
        verifylist = [
            ('instance', 'Instance1'),
            ('delete_instance', True)
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.remove_instance.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.remove_instance.assert_called_once()


class TestBatchAutoScalingInstanceAction(TestAutoScalingInstance):

    def setUp(self):
        super(TestBatchAutoScalingInstanceAction, self).setUp()

        self.cmd = instance.BatchActionAutoScalingInstance(self.app, None)

        self.client.batch_instance_action = mock.Mock()

    def test_wrong_action(self):
        arglist = [
            '--group', 'grp1',
            'ADD1',
            'Instance1',
            '--delete-instance',
        ]
        verifylist = [
            ('instance', ['Instance1']),
            ('delete_instance', True),
            ('action', 'ADD1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args
        )

    def test_add(self):
        arglist = [
            '--group', 'grp1',
            'ADD',
            'Instance1',
        ]
        verifylist = [
            ('instance', ['Instance1']),
            ('action', 'ADD')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.batch_instance_action.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.batch_instance_action.assert_called_with(
            action='ADD',
            delete_instance=False,
            group='grp1',
            ignore_missing=False, instance=['Instance1']
        )

    def test_remove(self):
        arglist = [
            '--group', 'grp1',
            'REMOVE',
            'Instance1',
            'Instance2',
            '--delete-instance',
        ]
        verifylist = [
            ('instance', ['Instance1', 'Instance2']),
            ('delete_instance', True),
            ('action', 'REMOVE')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.batch_instance_action.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.batch_instance_action.assert_called_with(
            action='REMOVE',
            delete_instance=True,
            group='grp1',
            ignore_missing=False, instance=['Instance1', 'Instance2']
        )

    def test_protect(self):
        arglist = [
            '--group', 'grp1',
            'protect',
            'Instance1',
            'Instance2',
        ]
        verifylist = [
            ('instance', ['Instance1', 'Instance2']),
            ('action', 'protect')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.batch_instance_action.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.batch_instance_action.assert_called_with(
            action='PROTECT',
            delete_instance=False,
            group='grp1',
            ignore_missing=False, instance=['Instance1', 'Instance2']
        )

    def test_unprotect(self):
        arglist = [
            '--group', 'grp1',
            'unProtect',
            'Instance1',
            'Instance2',
        ]
        verifylist = [
            ('instance', ['Instance1', 'Instance2']),
            ('action', 'unProtect')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.batch_instance_action.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.batch_instance_action.assert_called_with(
            action='UNPROTECT',
            delete_instance=False,
            group='grp1',
            ignore_missing=False, instance=['Instance1', 'Instance2']
        )
